/**
 * @class Sb.EcwidWidgetsManager
 * @singleton
 */
Ext.define('Sb.EcwidWidgetsManager', {
	singleton: true,

	isInvalidEshopId: false,

	widgetsCollection: new Ext.util.MixedCollection(false,
		function (config) {
			return config.id;
		}
	),

	onHashChange: function (evt) {
		var hash = window.location.hash,
			reEcwidHash = /(.)+ecwid:(.)+/;
		if (reEcwidHash.test(hash)) {
			var widgetType = Sb.WidgetTypeManager.get('eshopCatalog');
			Sb.Editor.siteView.setCurrentPage(widgetType.getPage(), function () {
				window.location.hash = hash;
			});
		}
	},

	register: function (config) {
		this.widgetsCollection.add(config);
		var l = 0;
		if (!window._xnext_initialization_scripts) {
			window._xnext_initialization_scripts = [];
		} else {
			l = window._xnext_initialization_scripts.length;
		}

		Ext.each(window._xnext_initialization_scripts, function (widgetConfig, index) {
			if (widgetConfig.id == config.id) {
				l = index;
				return false;
			}
		});
		window._xnext_initialization_scripts[l] = config;
	},

	unregister: function (config) {
		if (config.widgetType == 'ProductBrowser') {
			config.widget.model.on('remove', this.defaultWidgetContent, this);
		}
		this.widgetsCollection.removeKey(config.id);
		Ext.each(window._xnext_initialization_scripts, function (widgetConfig, index) {
			if (widgetConfig.id == config.id) {
				window._xnext_initialization_scripts.remove(widgetConfig);
				return false;
			}
		});
	},

	clear: function () {
		this.widgetsCollection.clear();
		window._xnext_initialization_scripts = [];
	},

	defaultWidgetContent: function () {
		this.widgetsCollection.each(function (config) {
			config.widget.renderPlaceholder();
		});
	},

	isProductBrowserWidget: function () {
		var widgetType = Sb.WidgetTypeManager.get('eshopCatalog');
		return !!widgetType.getPage();
	},

	getProductBrowserWidget: function () {
		var widgetConfig = null;
		if (this.isProductBrowserWidget) {
			widgetConfig = this.widgetsCollection.find(function (item) {
				if (item.widgetType == 'ProductBrowser') {
					return true;
				}
			});
		}
		if (widgetConfig) {
			return widgetConfig.widget;
		}
		return null;
	},

	onRefresh: function (callback) {
		this.showLoader(Sb.Locale.get('eshopLoading'));
		this.widgetsCollection.each(function (config) {
			config.widget.refreshContent();
		});
		window.ecwid_ProductBrowserURL = Sb.Config.baseUrl + '/Editor#';
		ecwid_bodyDone = true;
		if (this.isProductBrowserWidget()) {
			ecwid_bodyDone = false;
			ecwid_onBodyDone();
		} else {
			this.defaultWidgetContent();
			this.hideLoader();
		}
		Ext.callback(callback);
	},

	getEshopId: function () {
		return window.xnext_ownerId || Sb.Editor.model.get('ecwidStoreId');
	},

	showLoader: function (msg) {
		Ext.get('ecwid_loading_indicator-msg').update(msg);
		Ext.get('ecwid_loading_indicator').show();
	},

	hideLoader: function () {
		Ext.get('ecwid_loading_indicator').hide();
	},

	onContentLoaded: function () {
		this.widgetsCollection.each(function (config, index) {
			config.widget.onContentLoaded();
		});
	},

	/**
	 * Refresh content of ecwid widgets
	 * @param {Boolean} [force] Reload a ecwid eshop owner script or not. Default false
	 * @param {Function} [callback] Function executed after refresh
	 */
	refresh: function (force, callback) {
		force = !!(force);
		if (!this.widgetsCollection.getCount()) {
			return;
		}

		if (force) {
			this.initEcwidAPI(this.getEshopId(), function () {
				this.refresh(false, callback);
			}, this, force);
			return;
		}

		if (window.Ecwid) {
			this.onRefresh(callback);
		} else {
			var ecwidStoreId = Sb.Editor.model.get('ecwidStoreId');
			if (ecwidStoreId) {
				this.showLoader(Sb.Locale.get('eshopLoadingScript'));
				this.isInvalidEshopId = false;

				this.initEcwidAPI(ecwidStoreId, function () {
					if (window.Ecwid) {
						if (Ecwid.OnPageLoaded) {
							Ecwid.OnPageLoaded.add(
								Ext.createDelegate(this.onContentLoaded, this)
							);
						}
						this.onRefresh(callback);
					} else {
						this.onEcwidLoadError();
					}
				}, this);

			} else {
				this.defaultWidgetContent();
			}
		}
	},

	/**
	 * @param {Number} storeId
	 * @param {Function} callback
	 * @param {Object} scope
	 * @param {Boolean} [force]
	 */
	initEcwidAPI: function (storeId, callback, scope, force) {
		force = force || false;
		window.xnext_ownerId = storeId;
		window.css_selectors_prefix = '';
		window.ecwid_nocssrewrite = true;
		window.ecwid_use_custom_loading_indicator = true;
		window.ecwid_script_defer = true;
		if (force) {
			if (Ext.isIE) {
				window.Ecwid = null;
			} else {
				delete window.Ecwid;
			}
			window._xnext_included = false;
		}
		Sb.ResourceManager.require([{
			url: Sb.Config.ecwidAppUrl + '/script.js?' + storeId + '&lang=' +
				Sb.Editor.siteView.getCurrentSiteLocale().substr(0, 2),
			type: 'js',
			force: force,
			scope: this,
			onError: this.onEcwidLoadError
		}], callback, scope);
	},

	onEcwidLoadError: function () {
		this.isInvalidEshopId = true;
		this.hideLoader();
		Sb.Editor.Ui.mainWindow.statusBar.setMessageError(Sb.Locale.get('eshopLoadingError', this.getEshopId()));
		this.defaultWidgetContent();
	}
});