/**
 * @class Sb.ModelDependencyManager
 * <p>Manages dependencies between models, when we can update dependent properties automatically.</p>
 * @singleton
 */
Ext.define('Sb.ModelDependencyManager', {
	singleton: true,

	/**
	 * Create properties dependency between generator and dependent with optional help of component.
	 *
	 * @param {Sb.model.Node} generator
	 * @param {String} generatorProperty
	 * @param {Sb.model.Node} dependent
	 * @param {String} dependentProperty
	 */
	attach: function (generator, generatorProperty, dependent, dependentProperty) {
		if (!dependent.isDependentProperty(dependentProperty)) {
			return;
		}

		function updateDependentProperty(key, value, oldValue) {
			if (key != generatorProperty) {
				return;
			}
			dependent.updateDependentProperty(dependentProperty, value);
		}
		generator.on('propertychange', updateDependentProperty, generator);

		function saveDependentObject() {
			dependent.saveIfChangedAsDependent();
		}
		generator.on('save', saveDependentObject, generator);

		function removeDependency() {
			generator.un('propertychange', updateDependentProperty, generator);
			generator.un('save', saveDependentObject, generator);
		}
		generator.on('remove', removeDependency, generator);
		dependent.on('remove', removeDependency, dependent);
		Sb.Editor.model.on('beforeload', removeDependency);
	}
});