/**
 * @class Sb.StyleManager
 * Utility class for manipulating SiteBuilder's CSS styles
 * @singleton
 */
Ext.define('Sb.StyleManager', function () {
	var cache = {},
		tasks = {},
		buffer = 250; // ms

	// Combines the styles of one section, and inserts them into the DOM
	function combineAndAdd(id) {
		var css = '',
			styleElement;
		Ext.iterate(cache[id], function (subId, cssText) {
			css += cssText;
		});

		id = 'sb-style-' + id;
		Ext.util.CSS.removeStyleSheet(id);

		if (Ext.isIE11) {
			styleElement = document.createElement('style');
			styleElement.type = 'text/css';
			styleElement.id = id;
			styleElement.appendChild(document.createTextNode(css));
			document.getElementsByTagName('head')[0].appendChild(styleElement);
			Ext.util.CSS.cacheStyleSheet(document.styleSheets[document.styleSheets.length-1]);
		} else {
			Ext.util.CSS.createStyleSheet(css, id);
		}
	}

	return {
		singleton: true,

		addStyleSheet: function (cssText, id, subId) {
			subId = subId || id;
			if (!cache[id]) {
				cache[id] = {};
			}
			cache[id][subId] = cssText;

			if (buffer > 0) {
				if (!tasks[id]) {
					tasks[id] = new Ext.util.DelayedTask(function () {
						combineAndAdd(id);
						delete tasks[id];
					});
					tasks[id].delay(buffer);
				}
			} else {
				combineAndAdd(id);
			}
		},

		getStyleSheet: function (id, subId) {
			if (subId) {
				return cache[id][subId];
			}

			var css = '';
			Ext.iterate(cache[id], function (subId, cssText) {
				css += cssText;
			});

			return css;
		}
	};
});
