Ext.onReady(function () {
	Ext.BLANK_IMAGE_URL = Sb.Config.blankImageUrl;
	Ext.QuickTips.init();

	var selectedTemplateId = null;

	var templatesStore = new Ext.data.DirectStore({
		directFn: Sb.SiteTemplateService.getList,
		autoLoad: true,
		fields: ['id', 'title', 'description', {name: 'templates', type: 'object'}],
		listeners: {
			load: function () {
				doLayout();
				categoriesView.select(0);
				Sb.Loading.hide();
				showOnLoadDialog();
				showImportDialog();
			}
		},
		filterByKeywords: function (query) {
			this.snapshot = this.snapshot || this.data;
			var data = this.snapshot || this.data;

			var sep = new RegExp('[~!@#%^&:%_, <>\\-\\/]');
			query = query.trim().split(sep);
			Ext.iterate(query, function (word, index) {
				query[index] = Ext.escapeRe(word);
			}, query);
			var queryRegExp = new RegExp('^(' + query.join('|') + ')', 'ig');

			var r = new Ext.util.MixedCollection();
			r.getKey = data.getKey;
			var keys = data.keys, it = data.items;
			var count = 0;
			for (var i = 0, len = it.length; i < len; i++) {
				var templates = it[i].get('templates');
				var matchedTemplates = [];
				for (var j = 0, lenj = templates.length; j < lenj; j++) {
					var match = false;

					for (var k = 0, lenk = templates[j].keywords.length; k < lenk; k++) {
						if (queryRegExp.test(templates[j].keywords[k])) {
							match = true;
							break;
						}
					}

					var words = templates[j].description.split(sep).concat(templates[j].title.split(sep));
					for (var idescr = 0; idescr < words.length; idescr++) {
						if (queryRegExp.test(words[idescr])) {
							match = true;
							break;
						}
					}

					if (match) {
						matchedTemplates.push(templates[j]);
					}
				}
				count = count + matchedTemplates.length;
				r.add(keys[i], new Ext.data.Record({
					id: it[i].get('id'),
					title: it[i].get('title'),
					description: it[i].get('description'),
					templates: matchedTemplates
				}));
			}

			this.data = r;
			this.fireEvent('datachanged', this);
			return count;
		}
	});

	var categoriesView = new Ext.DataView({
		store: templatesStore,
		singleSelect: true,
		renderTo: 'wizard-categories-box',
		tpl: new Ext.XTemplate(
			'<ul class="wizard-categories-box">',
				'<tpl for=".">',
				'<li class="cat-{id} {[values.templates.length > 0 ? "" : "empty"]}"><a href="#"><span><b>{title}{[values.templates.length > 0 ? " (" + values.templates.length + ")" : ""]}</b></span></a></li>',
				'</tpl>',
			'</ul>'
		),
		itemSelector: 'li',
		selectedClass: 'active',
		listeners: {
			selectionchange: function (dv) {
				if (dv.getSelectionCount() < 1) {
					return;
				}

				if (bound) {
					var index = dv.getSelectedIndexes()[0];
					var categoryElement = Ext.fly('category-' + (index + 1));
					categoryElement.removeClass('collapsed');
					var top = categoryElement.dom.offsetTop + 1;
					unbindScroll();
					templatesView.container.scrollTo('top', top, {callback: bindScroll});
				}
			},
			beforeclick: function (dv, index, node, e) {
				if (Ext.get(node).hasClass('empty')) {
					return false;
				}
			}
		}
	});

	var templatesView = new Ext.DataView({
		store: templatesStore,
		singleSelect: true,
		renderTo: 'wizard-templates-box',
		tpl: new Ext.XTemplate(
			'<tpl for=".">',
			'<tpl if="templates.length &gt; 0">', // :NOTE: that the > is encoded
			'<div class="b-category" id="category-{#}">',
				'<h3 class="b-category-title">',
					'<span onClick="Ext.get(this).up(\'.b-category\').toggleClass(\'collapsed\');"><em></em>{title}</span>',
				'</h3>',
				'<div class="b-category-description">{description}</div>',

				'<ul class="wizard-templates-box">',
				'<tpl for="templates">',
					'<li class="" id="{id}">',
						'<div class="tpl-wrap-1"><div class="tpl-wrap-2"><div class="tpl-wrap-3"><div class="tpl-wrap-4">',
							'<div class="tpl-content">',
								'<div class="tpl-img" style="background: url(' + Sb.Utils.antiCache(Sb.Config.siteTemplatesUrl + '/{id}.png') + ') no-repeat 0 0;"></div>',
								'<div class="tpl-title"><h3><a href="#"><span>{title:wbrBySlash}</span></a></h3></div>',
								'<div class="tpl-description">{description:defaultValue("This template does not have a description yet.")}</div>',
							'</div>',
						'</div></div></div></div>',
					'</li>',
				'</tpl>',
				'</ul>',
			'</div>',
			'</tpl>',
			'</tpl>'
		),
		itemSelector: 'li',
		selectedClass: 'tpl-selected',
		overClass: 'tpl-hover',
		listeners: {
			containerclick: function () {
				return false;
			},
			beforeselect: function (dv,node) {
				selectedTemplateId = node.id;
				onCreateSite();
				return false;
			}
		}
	});

	var getHelpUrl = function (context) {
		return Sb.Config.helpUrl.replace('%%CONTEXT%%', encodeURIComponent('SiteBuilder' + context));
	};

	var startImportHandler = function () {
		var win = new Ext.Window({
			width: 500,
			title: Sb.Locale.get('ImportYourWebsite'),
			renderTo: Ext.getBody(),
			modal: true,
			draggable: false,
			resizable: false,
			shadow: false,
			buttonAlign: 'center',
			keys: {
				key: Ext.EventObject.ENTER,
				fn: function (keyCode, e) {
					e.stopPropagation();
					win.fbar.items.first().btnEl.dom.click();
				}
			},
			items: {
				xtype: 'form',
				id: 'sb-import-form',
				border: false,
				labelAlign: 'top',
				labelWidth: 100,
				defaults: {
					hideLabel: true,
					msgTarget: 'under'
				},
				items: [{
					xtype: 'displayfield',
					width: '100%',
					html: '<p>' + Sb.Locale.get('ImportSiteDescription', getHelpUrl('/site-import-limitations')) + '</p>'
				}, {
					xtype: 'textfield',
					vtype: 'url',
					validationEvent : 'change',
					fieldLabel: Sb.Locale.get('ImportSiteURL'),
					name: 'importSiteUrl',
					allowBlank: false,
					hideLabel: false,
					maxLength: 255,
					anchor: '100%',
					ctCls: 'sb-import-form-textfield',
					stripCharsRe: /\s*/g
				}, {
					xtype: 'checkbox',
					name: 'acceptLimitations',
					boxLabel: Sb.Locale.get('acceptImportLimitations', getHelpUrl('/site-import-limitations')),
					ctCls: 'sb-import-form-checkbox',
					listeners: {
						check: function (checkbox, isChecked) {
							win.fbar.items.first().setDisabled(!isChecked);
						}
					}
				}]
			},
			fbar: [{
				cls: 'sb-btn-action',
				text: Sb.Locale.get('ImportSiteButton'),
				disabled: true,
				handler: startImport
			}]
		});
		win.show();
	};

	function startImport() {
		var form = Ext.getCmp('sb-import-form').getForm();
		if (!form.isValid()) {
			return;
		}

		Sb.Loading.updateMessage(Sb.Locale.get('loaderMsgStartingImport'));
		Sb.Loading.show();

		// Increase AJAX timeout to 100 minutes
		var timeout = Sb.model.apiProvider.timeout;
		Sb.model.apiProvider.timeout *= 10;

		var task = {
			run: Sb.app.renewSessionHandler,
			interval: 60000, //1 minute,
			scope: Sb.app
		};
		var runner = new Ext.util.TaskRunner();
		runner.start(task);

		var url = form.findField('importSiteUrl').getValue();
		Sb.SiteService.importSite(url, function (result, response) {
			runner.stop(task);
			Sb.model.apiProvider.timeout = timeout;
			if (result && result.success) {
				window.location = Sb.Config.baseUrl + '/Editor';
			} else {
				Sb.Loading.hide();
				var msg = Sb.Locale.get('importSiteFailed', getHelpUrl('/site-import-failed'));
				if (result && result.message) {
					msg = result.message + '<br />' + msg;
				} else if (response && response.message) {
					msg = response.message + '<br/>' + msg;
				}
				Ext.Msg.show({
					title: Sb.Locale.get('errorMessageBoxTitle'),
					msg: msg,
					buttons: Ext.MessageBox.OK,
					closable: false
				});
			}
		});
	}

	if (Sb.Config.showImportSiteButton) {
		new Ext.Button({
			text: Sb.Locale.get('ImportSite'),
			renderTo: 'sb-import-button',
			handler: startImportHandler,
			disabled: Sb.Config.isDemo  || Sb.Config.isTrialMode,
			tooltip: Sb.Config.isDemo  || Sb.Config.isTrialMode ? Sb.Locale.get('importNotAvailableInDemo'): ''
		});
	}

	new Ext.form.TextField({
		applyTo: 'search-field',
		emptyText: Sb.Locale.get('EmptySearch'),
		value: '',
		lengthToSearch: 0,
		enableKeyEvents: true,
		listeners: {
			keyup: function (tf, e) {
				if (e.ESC == e.getKey()) {
					tf.setValue('');
					templatesStore.clearFilter();
				}

				if (tf.getValue().trim().length > this.lengthToSearch) {
					if (!templatesStore.filterByKeywords(tf.getValue())) {
						templatesView.update('<div class="b-category-empty">' + Sb.Locale.get('NoMatch') + '</div>');
					}
				} else {
					templatesStore.clearFilter();
				}

				calcScrollPosition();
			}
		}
	});

	function onCreateSite(){
		if (!siteOwnerInfo || isSiteDeploymentStarted || !selectedTemplateId) {
			return;
		}

		var selectedCategory = null;
		templatesStore.each(function (record) {
			Ext.each(record.get('templates'), function (template) {
				if (template.id == selectedTemplateId) {
					selectedCategory = record.get('id');
					return false;
				}
			});
			if (selectedCategory) {
				return false;
			}
		});

		if (selectedCategory == 'Other') {
			var win = new Ext.Window({
				id: 'submit-website-topic-suggest-win',
				width: 400,
				title: Sb.Locale.get('SubmitWebsiteTopicSuggestTitle'),
				renderTo: Ext.getBody(),
				modal: true,
				draggable: false,
				resizable: false,
				shadow: false,
				keys: {
					key: Ext.EventObject.ENTER,
					fn: function (keyCode, e) {
						e.stopPropagation();
						if (e.getTarget('textarea')) {
							return;
						}
						win.fbar.items.get(0).btnEl.dom.click();
					}
				},
				items: {
					xtype: 'form',
					border: false,
					items: [{
						xtype: 'displayfield',
						hideLabel: true,
						html: '<p>' + Sb.Locale.get('SubmitWebsiteTopicSuggestDescription') + '</p>'
					}, {
						emptyText: Sb.Locale.get('SubmitWebsiteTopicSuggestEmptyField'),
						id: 'submit-website-topic-suggest-text',
						hideLabel: true,
						xtype: 'textarea',
						maxLength: 1000,
						anchor: '100%'
					}]
				},
				buttonAlign: 'center',
				fbar: [{
					cls: 'sb-btn-action',
					text: Sb.Locale.get('SubmitAndCreateSite'),
					handler: function () {
						Sb.SiteTemplateService.submitSuggest(Ext.getCmp('submit-website-topic-suggest-text').getValue().substring(0, 1000));
						showUserInfoDialog();
						win.close();
					}
				}]
			});
			win.show();
			return;
		}

		showUserInfoDialog();
	}

	var isSiteDeploymentStarted = false;

	function preloadResources() {
		var preload = Ext.emptyFn;
		Ext.each(Sb.cssListForPreloading.concat(Sb.jsListForPreloading), function (fileUrl) {
			var preloadSequence = preload;
			preload = function () {
				if (isSiteDeploymentStarted) {
					return;
				}
				Ext.Ajax.request({
					url: fileUrl,
					disableCaching: false,
					callback: preloadSequence
				});
			};
		});
		preload();
	}

	var siteOwnerInfo = {};
	Sb.SiteService.getOwnerInfo(
		function (result, response) {
			if (response.status) {
				siteOwnerInfo = result;
			}
			preloadResources.defer(2000);
		}
	);

	var localesData = [],
		localesStore;

	Ext.iterate(Sb.Config.locales, function (code, data) {
		if (!data.hasTranslation) {
			return true;
		}
		localesData.push([
			code,
			data.name,
			'background-image: url(' + Sb.Config.siteResourcesUrl + '/images/flags/16/' +
				code.split('_')[1].toLowerCase() + '.png);'
		]);
	});

	localesStore = new Ext.data.ArrayStore({
		fields: [
			'code',
			'name',
			'iconStyle'
		],
		data: localesData,
		sortInfo: {
			field: 'name'
		}
	});

	function updateUpsellUi() {
		var form = Ext.getCmp('userEditInfoForm'),
			locales = Ext.unique([].concat(form.getForm().getValues().locales)),
			isExceeded = Sb.maxAdditionalLocales !== -1 && (locales.length - 1) > Sb.maxAdditionalLocales;

		Ext.getCmp('btnCreateSite')[isExceeded ? 'addClass' : 'removeClass']('sb-upsell-button');

		locales = [];
		form.findByType('compositefield').forEach(function (cmp) {
			var locale = cmp.combo.getValue(),
				isUpsell = isExceeded && locales.length > Sb.maxAdditionalLocales && locales.indexOf(locale) === -1,
				child;

			if (locales.indexOf(locale) === -1) {
				locales.push(locale);
			}

			child = cmp.getEl().child('.sb-locale-upsell-message');
			if (isUpsell && !child) {
				cmp.getEl().createChild({
					cls: 'sb-locale-upsell-icon',
					title: Sb.Locale.get('additionalLocaleUpsellMessage')
				});
			} else if (!isUpsell && child) {
				child.remove();
			}
		});
	}

	function getLocalesComboConfig(actionType, fieldLabel) {
		var btn, cfg;

		if (Sb.Config.isEnabledMlw) {
			btn = {
				xtype: 'button',
				iconCls: 'sb-add-icon',
				width: 24,
				height: 24,
				handler: function () {
					var form = this.findParentByType('form'),
						fields = form.findByType('compositefield');

					form.insert(2 + fields.length, getLocalesComboConfig('remove'));
					form.doLayout();
					updateUpsellUi();
				}
			};

			if (actionType !== 'add') {
				btn = Ext.apply(btn, {
					iconCls: 'sb-delete-icon',
					handler: function () {
						var field = this.findParentByType('compositefield'),
							parentForm = this.findParentByType('form');

						parentForm.getForm().remove(field.combo);
						parentForm.remove(field, true);
						updateUpsellUi();
					}
				});
			}
		}

		fieldLabel = fieldLabel || '';

		cfg = {
			xtype: 'compositefield',
			itemCls: 'sb-field-locale',
			defaults: {
				flex: 1
			},
			items: [{
				xtype: 'iconcombo',
				ref: 'combo',
				editable: false,
				triggerAction: 'all',
				mode: 'local',
				shadow: false,
				store: localesStore,
				hiddenName: 'locales',
				valueField: 'code',
				displayField: 'name',
				iconStyleField: 'iconStyle',
				value: Sb.Config.locale,
				fieldLabel: fieldLabel,
				listeners: {
					select: updateUpsellUi
				}
			}]
		};

		if (btn) {
			cfg.items.push(btn);
		}

		return cfg;
	}

	function showUserInfoDialog() {
		var win = new Ext.Window({
			autoScroll: true,
			width: 505,
			height: 613,
			title: Sb.Locale.get('UserEditInfoTitle'),
			renderTo: Ext.getBody(),
			modal: true,
			draggable: false,
			resizable: false,
			shadow: false,
			keys: {
				key: Ext.EventObject.ENTER,
				fn: function (keyCode, e) {
					e.stopPropagation();
					if (e.getTarget('textarea')) {
						return;
					}
					win.fbar.items.get(0).btnEl.dom.click();
				}
			},
			items: {
				xtype: 'form',
				border: false,
				id: 'userEditInfoForm',
				labelAlign: 'left',
				labelWidth: 175,
				defaults: {
					xtype: 'textfield',
					maxLength: 255,
					width: 280,
					msgTarget: 'under'
				},
				items: [{
					xtype: 'box',
					width: '100%',
					autoEl: {
						tag: 'p',
						html: Sb.Locale.get('UserEditInfoDescription')
					}
				}, {
					xtype: 'textfield',
					name: 'siteName',
					maxLength: 255,
					value: Sb.Locale.get('defaultSiteName'),
					fieldLabel: Sb.Locale.get('SiteName')
				}, getLocalesComboConfig('add', Sb.Locale.get('SiteLanguage')), {
					fieldLabel: Sb.Locale.get('siteOwnerInfoPersonalName'),
					name: 'personalName',
					value: siteOwnerInfo.personalName
				}, {
					fieldLabel: Sb.Locale.get('siteOwnerInfoCompanyName'),
					name: 'companyName',
					value: siteOwnerInfo.companyName
				}, {
					fieldLabel: Sb.Locale.get('siteOwnerInfoPhone'),
					name: 'phone',
					value: siteOwnerInfo.phone
				}, {
					fieldLabel: Sb.Locale.get('siteOwnerInfoEmail'),
					name: 'email',
					value: siteOwnerInfo.email,
					vtype: 'email',
					validationEvent : 'change'
				}, {
					xtype: 'textarea',
					height: 52,
					fieldLabel: Sb.Locale.get('siteOwnerInfoAddress'),
					name: 'address',
					value: siteOwnerInfo.address
				}, {
					fieldLabel: Sb.Locale.get('siteOwnerInfoCity'),
					name: 'city',
					value: siteOwnerInfo.city
				}, {
					fieldLabel: Sb.Locale.get('siteOwnerInfoState'),
					name: 'state',
					value: siteOwnerInfo.state
				}, {
					fieldLabel: Sb.Locale.get('siteOwnerInfoZip'),
					name: 'zip',
					value: siteOwnerInfo.zip
				}, {
					fieldLabel: Sb.Locale.get('siteOwnerInfoCountry'),
					name: 'country',
					value: siteOwnerInfo.country
				}]
			},
			buttonAlign: 'center',
			fbar: [{
				cls: 'sb-btn-action',
				id: 'btnCreateSite',
				text: Sb.Locale.get('SubmitAndCreateSite'),
				handler: function () {
					if (!Ext.getCmp('userEditInfoForm').getForm().isValid()) {
						return;
					}
					deploySiteTemplate();
					win.close();
				}
			}]
		});
		win.show();
	}

	var deploySiteTemplate = function () {
		Sb.Loading.updateMessage(Sb.Locale.get('loaderMsgSiteCreated'));
		Sb.Loading.show();
		isSiteDeploymentStarted = true;

		var formValues = Ext.getCmp('userEditInfoForm').getForm().getValues(),
			siteName = formValues.siteName,
			locales = [].concat(formValues.locales);

		delete formValues.locales;
		delete formValues.siteName;

		Sb.SiteService.deploySiteTemplate(
			selectedTemplateId,
			locales[0],
			siteName,
			formValues,
			locales,
			function (result, response) {
				if (response.status) {
					window.location = Sb.Config.baseUrl + '/Editor';
				}
			}
		);
	};

	var doLayout = function () {
		var scrollArea = Ext.get('wizard-templates-box');
		var distanceToTop = scrollArea.getOffsetsTo(Ext.getBody());
		var footerAreaHeight = Ext.get('footer-wrapper').getHeight();
		var bodyHeight = Ext.getBody().getHeight();

		var minScrollAreaHeight = Ext.fly('wizard-categories-box').getHeight();
		var newScrollAreaHeight = parseInt(bodyHeight, 10) - parseInt(distanceToTop[1], 10) - parseInt(footerAreaHeight, 10) - 20;
		if (newScrollAreaHeight < minScrollAreaHeight) {
			newScrollAreaHeight = minScrollAreaHeight;
		}

		if (scrollArea.getHeight() != newScrollAreaHeight) {
			scrollArea.setHeight(newScrollAreaHeight);
		}

		var mainContainer = Ext.get('page');
		var contentMinHeight = Ext.get('wrapper').getHeight();
		if (bodyHeight > contentMinHeight) {
			mainContainer.setHeight(bodyHeight);
		} else {
			mainContainer.setHeight();
		}
	};
	Ext.EventManager.onWindowResize(doLayout);

	var bound = true;
	function bindScroll() {
		templatesView.container.on('scroll', calcScrollPosition, templatesView.container, {buffer: 250});
		bound = true;
	}
	function unbindScroll() {
		templatesView.container.un('scroll', calcScrollPosition, templatesView.container);
		bound = false;
	}

	function calcScrollPosition() {
		var lastId = 0;
		templatesView.container.select('.b-category').each(function (el) {
			if (el.getOffsetsTo(templatesView.container)[1] > 200) {
				return false;
			}
			lastId = parseInt(el.id.substr(9), 10) - 1;
		});
		categoriesView.suspendEvents();
		categoriesView.select(lastId);
		categoriesView.resumeEvents();
	}

	doLayout();

	function showOnLoadDialog() {
		var placeholder = Ext.get('sb-dialog-onload');
		if (null == placeholder) {
			return;
		}

		var title = placeholder.first('.sb-dialog-header').getAttribute('innerHTML'),
			body = placeholder.first('.sb-dialog-body').getAttribute('innerHTML'),
			buttonText = placeholder.first('.sb-dialog-button').getAttribute('innerHTML');

		var win = new Ext.Window({
			width: 500,
			modal: true,
			resizable: false,
			draggable: false,
			renderTo: Ext.getBody(),
			title: title,
			items: {
				xtype: 'box',
				html: body
			},
			fbar: [{
				cls: 'sb-btn-action',
				text: buttonText,
				handler: function () {
					win.close();
				}
			}]
		});
		win.show();
	}

	function showImportDialog() {
		var importFrom = Sb.Utils.getQueryParamValue('importFrom');
		if (importFrom || self.location.search.indexOf('?import') === 0) {
			startImportHandler();
			if (importFrom) {
				var form = Ext.getCmp('sb-import-form').getForm();
				form.findField('importSiteUrl').setValue(importFrom);
				startImport();
			}
		}
	}
});