/**
 * @class Sb.dashboard.Model
 * @extends Ext.util.Observable
 * Base class for dashboard tile models
 */
Ext.define('Sb.dashboard.Model', {
	extend: 'Ext.util.Observable',

	/**
	 * @type String
	 */
	name: 'tile',

	/**
	 * @type string
	 */
	lastError: '',

	/**
	 * Settings are stored in the db
	 * @type Object
	 */
	settings: {},

	/**
	 * Loaded data
	 */
	data: undefined,

	/**
	 * @type Ext.util.TaskRunner
	 */
	runner: undefined,

	/**
	 * List of tasks
	 */
	tasks: {},

	/**
	 * List names of running tasks
	 * @see startTask
	 * @see stopTask
	 */
	runningTasks: {},

	constructor: function (config) {
		config = config || {};
		this.addEvents('settingsload', 'datachanged', 'error');

		this.settings = config.settings || {};
		this.tasks = config.tasks || {};
		this.runningTasks = {};
		this.runner = new Ext.util.TaskRunner();

		Sb.dashboard.Model.superclass.constructor.call(this, config);
	},

	/**
	 * @param {String} name Key from task list
	 * @param {Boolean} forceRun
	 */
	startTask: function (name, forceRun) {
		if (!this.tasks[name]) {
			return false;
		}

		if (!this.runningTasks[name]) {
			this.runningTasks[name] = true;
			this.runner.start(this.tasks[name]);
		} else if (forceRun) {
			this.tasks[name].run();
		}
	},

	/**
	 * @param {String} name  Key from task list
	 */
	stopTask: function (name) {
		if (this.tasks[name]) {
			this.runningTasks[name] = false;
			this.runner.stop(this.tasks[name]);
		}
	},

	/**
	 * Get setting value
	 * @param name
	 */
	get: function (name) {
		return this.settings[name];
	},

	/**
	 * Set setting value
	 * @param name
	 * @param value
	 */
	set: function (name, value) {
		this.settings[name] = value;
	},

	loadSettings: function () {
		var propertiesName = [];
		Ext.iterate(this.settings, function (name) {
			propertiesName.push(name);
		});
		Sb.DashboardService.getTileSettings(this.name, propertiesName, function (result) {
			if (result.success) {
				Ext.apply(this.settings, result.data);
				this.fireEvent('settingsload', this);
			} else {
				this.lastError = result.message || Sb.Locale.get('unknownLoadingError');
			}
		}, this);
	},

	saveSettings: function () {
		Sb.DashboardService.setTileSettings(this.name, this.settings);
	},

	isEnabled: function () {
		return true;
	}
});