/**
 * @class Sb.dashboard.Tile
 * @extends Ext.Panel
 * Base class for dashboard tiles
 * @xtype sb.dashboard.tile
 */
Ext.define('Sb.dashboard.Tile', {
	extend: 'Ext.Panel',
	xtype: 'sb.dashboard.tile',

	cls: 'sb-dashboard-tile sb-dashboard-tile-small',
	logoCls: '',
	frame: true,
	titleString: '',

	/**
	 * @type Sb.dashboard.Manager
	 */
	manager: undefined,

	/**
	 * @type Sb.dashboard.Model
	 */
	model: undefined,

	initComponent: function () {
		if (!this.items) {
			Ext.apply(this, {
				bodyCssClass: 'sb-dashboard-tile-fix-margin',
				items: [{
					xtype: 'container',
					layout: {
						type: 'table',
						align: 'top',
						columns: 2,
						tableAttrs: {
							cls: 'sb-dashboard-width100'
						}
					},
					items: [{
						xtype: 'box',
						layout: 'fit',
						html: '<div class="sb-dashboard-tile-logo ' +  this.logoCls + '"></div>',
						height: '100%',
						cellCls: 'sb-dashboard-tile-logo-td'
					}, {
						xtype: 'container',
						width: '100%',
						cellCls: 'sb-dashboard-without-bottom-padding',
						items: [{
							xtype: 'container',
							ref: '../../titleBox',
							items: {
								xtype: 'box',
								html: '<h1 class="sb-dashboard-tile-title">' + this.titleString + '</h1>'
							}
						}, {
							xtype: 'container',
							ref: '../../contentBox',
							cls: 'sb-dashboard-tile-content'
						}]
					}]
				}]
			});
		}

		Sb.dashboard.Tile.superclass.initComponent.call(this);
	},

	/**
	 * Returns content container
	 * @return {Ext.Container}
	 */
	getContentEl: function () {
		return this.contentBox;
	},

	showMessage: function (message) {
		var el = this.getContentEl();
		el.removeAll();
		el.add({
			xtype: 'label',
			text: message
		});
		el.doLayout();
	},

	showError: function (message, type) {
		type = type || 'error';
		this.showMessage(message, type);
	},

	errorHandler: function (model, message, type) {
		this.showError(message, type);
	}

});