/**
 * @class Sb.dashboard.Window
 * @extends Sb.Window
 */
Ext.define('Sb.dashboard.Window', {
	extend: 'Sb.Window',

	modal: true,

	resizable: false,
	autoScroll: true,

	showAnalyticsTab: true,
	showNotificationTab: true,
	showExternalServicesTab: true,

	width: 820,
	height: 704,

	initComponent: function () {
		var tabsItemsConfig = [{
			tabCls: 'sb-dashboard-main-tab sb-dashboard-main-tab-first',
			iconCls: 'sb-dashboard-icon-tab-overview',
			ref: 'overviewTab',
			autoHeight: true,
			cls: 'sb-dashboard-overview',
			layout: 'auto',
			tbarCfg: {
				cls: 'sb-dashboard-overview-top'
			},
			tbar: [{
				cls: 'sb-dashboard-overview-title',
				xtype: 'label',
				text: Sb.Locale.get('overview')
			}, '->', {
				cls: 'sb-dashboard-checkbox-show',
				id: 'isShowDashboard',
				xtype: 'checkbox',
				checked: Ext.state.Manager.get('isShowDashboard'),
				boxLabel: Sb.Locale.get('isShowDashboard')
			}]
		}];

		if (this.showAnalyticsTab) {
			tabsItemsConfig.push({
				iconCls: 'sb-dashboard-icon-tab-analytics',
				ref: 'googleAnalyticsTab'
			});
		}

		if (this.showNotificationTab) {
			tabsItemsConfig.push({
				iconCls: 'sb-dashboard-icon-tab-notification',
				ref: 'notificationTab'
			});
		}

		if (this.showExternalServicesTab) {
			tabsItemsConfig.push({
				tabCls: 'sb-dashboard-main-tab sb-dashboard-main-tab-last',
				iconCls: 'sb-dashboard-icon-tab-external',
				ref: 'externalServicesTab'
			});
		} else {
			tabsItemsConfig[tabsItemsConfig.length - 1].tabCls = 'sb-dashboard-main-tab sb-dashboard-main-tab-last';
		}

		Ext.apply(this, {
			id: 'dashboardWindow',
			cls: 'sb-dashboard',
			header: false,
			headerCfg: {
				html: ''
			},
			closable: false,
			bbar: ['->', {
				text: Sb.Locale.get('closeDashboard').toUpperCase(),
				cls: 'sb-dashboard-btn-close-window',
				iconCls: 'sb-dashboard-icon-close-window',
				iconAlign: 'right',
				scope: this,
				handler: this.closeHandler
			}],
			items: {
				ref: 'tabs',
				xtype: 'tabpanel',
				activeTab: 0,
				headerCfg: {
					cls: 'sb-dashboard-main-tabs'
				},
				defaults: {
					tabCls: 'sb-dashboard-main-tab'
				},
				items: tabsItemsConfig,
				listeners: {
					beforetabchange: function (tabPanel, newTab, currentTab) {
						var tabs = [{
							tab: currentTab,
							action: 'removeClass'
						}, {
							tab: newTab,
							action: 'addClass'
						}];

						Ext.each(tabs, function (tabConfig) {
							var currentTabIndex = tabPanel.items.indexOf(tabConfig.tab);

							if (currentTabIndex > 0) {
								var prevTabIndex = currentTabIndex - 1,
									tabDomEl = tabPanel.getTabEl(prevTabIndex),
									tabEl = new Ext.Element(tabDomEl);

								tabEl[tabConfig.action]('sb-dashboard-main-tab-before-active');
							}
						});


					}
				}
			}
		});
		Sb.dashboard.Window.superclass.initComponent.call(this);
	},

	afterRender: function () {
		Sb.dashboard.Window.superclass.afterRender.apply(this, arguments);

		var windowHeight = Ext.getBody().getViewSize().height,
			margin = 20,
			minDashboardHeight = this.getHeight() + margin;

		if (windowHeight < minDashboardHeight) {
			var diffHeight = minDashboardHeight - windowHeight;
			this.setHeight(this.getHeight() - diffHeight);
		}
	},

	closeHandler: function () {
		Ext.state.Manager.set('isShowDashboard', Ext.getCmp('isShowDashboard').checked);
		this.close();
	}

});