/**
 * @class Sb.dashboard.model.GoogleAnalytics
 * @extends Sb.dashboard.Model
 * Google analytics model
 */
Ext.define('Sb.dashboard.model.GoogleAnalytics', {
	extend: 'Sb.dashboard.Model',

	name: 'analytics',

	currentMainMetric: undefined,

	extendedData: undefined,

	startDate: undefined,

	endDate: undefined,

	isLogged: false,

	metrics: [{
		id: 'ga:visits',
		label: Sb.Locale.get('gaVisits'),
		indicatorLabel: 'gaVisitsIndicator'
	}, {
		id: 'ga:pageviews',
		label: Sb.Locale.get('gaPageViews'),
		indicatorLabel: 'gaPageViewsIndicator'
	}, {
		id: 'ga:newVisits',
		label: Sb.Locale.get('gaNewVisits'),
		indicatorLabel: 'gaNewVisitsIndicator'
	}, {
		id: 'ga:pageviewsPerVisit',
		label: Sb.Locale.get('gaPageViewsPerVisit'),
		indicatorLabel: 'gaPageViewsPerVisitIndicator'
	}, {
		id: 'ga:avgTimeOnSite',
		label: Sb.Locale.get('gaAvgTimeOnSite'),
		indicatorLabel: 'gaAvgTimeOnSiteIndicator'
	}, {
		id: 'ga:percentNewVisits',
		label: Sb.Locale.get('gaPercentNewVisits'),
		indicatorLabel: 'gaPercentNewVisitsIndicator'
	}],

	constructor: function (config) {
		this.currentMainMetric = this.metrics[0];
		this.endDate = new Date().add(Date.DAY, -1);
		this.startDate = this.endDate.add(Date.MONTH, -1);
		this.addEvents('loginstatechange');

		config = config || {};
		config.settings = {
			tableId: undefined
		};
		config.tasks = {
			loadData: {
				run: this.loadData.createDelegate(this),
				interval: 1000 * 60 * 60 * 24 // 24 hours
			}
		};

		Sb.dashboard.model.GoogleAnalytics.superclass.constructor.call(this, config);

		Sb.ResourceManager.require([{url: '//www.google.com/jsapi', type: 'js'}], function () {
			google.load('visualization', '1', {
				packages: ['corechart', 'geochart'],
				callback: this.googleLoadCompleteHandler.createDelegate(this)
			});
		}, this);
	},

	googleLoadCompleteHandler: function () {
		this.loadSettings();
	},

	loadData: function () {
		var tableId = this.get('tableId');

		if (tableId) {
			var isExtended = this.extendedData === undefined;
			Sb.GoogleAnalyticsService.getData(
				tableId,
				this.startDate.format('Y-m-d'),
				this.endDate.format('Y-m-d'),
				this.currentMainMetric.id,
				Sb.Config.locale,
				isExtended,
				function (result, response) {
					if (result.success) {
						this.data = result.data;
						if (isExtended) {
							this.extendedData = result.extendedData;
						}
						this.fireEvent('datachanged', this);
					} else {
						this.lastError = result.message || Sb.Locale.get('unknownLoadingError');
						this.fireEvent('error', this, this.lastError, 'warning');
					}
				},
				this
			);
		}
	},

	toggleLoginState: function (isLogged) {
		if (isLogged != this.isLogged) {
			this.isLogged = isLogged;
			if (!isLogged) {
				Ext.iterate(Ext.Direct.transactions, function (i, t) {
					if ('GoogleAnalyticsService' == t.action) {
						Ext.Direct.removeTransaction(t);
					}
				});
				this.set('tableId', '');
				this.data = undefined;
				this.extendedData = undefined;
				this.saveSettings();
			}
			this.fireEvent('loginstatechange', this, this.isLogged);
		}
	}

});