/**
 * @class Sb.dashboard.tile.GoogleAnalyticsTile
 * @extends Sb.dashboard.Tile
 * Class for Google analytics tile
 * @xtype sb.dashboard.tile.ga
 */
Ext.define('Sb.dashboard.tile.GoogleAnalyticsTile', {
	extend: 'Sb.dashboard.Tile',
	xtype: 'sb.dashboard.tile.ga',

	height: 300,
	profiles: [],

	initComponent: function () {
		this.model = this.manager.getModel('ga');
		this.mon(this.model, 'datachanged', this.dataChangeHandler, this);
		this.mon(this.model, 'error', this.errorHandler, this);
		this.mon(this.model, 'loginstatechange', this.loginStateChangeHandler, this);
		this.mon(this.model, 'settingsload', this.settingsLoadHandler, this);

		Ext.apply(this, {
			cls: 'sb-dashboard-tile sb-dashboard-tile-big',
			bodyCssClass: 'sb-dashboard-tile-fix-margin',
			items: [{
				xtype: 'container',
				layout: {
					type: 'table',
					align: 'top',
					columns: 2,
					tableAttrs: {
						cls: 'sb-dashboard-width100'
					}
				},
				items: [{
					xtype: 'box',
					layout: 'fit',
					html: '<div class="sb-dashboard-tile-logo sb-dashboard-logo-analytics"></div>',
					height: '100%',
					cellCls: 'sb-dashboard-tile-logo-td'
				}, {
					xtype: 'container',
					width: '100%',
					cellCls: 'sb-dashboard-without-bottom-padding',
					items: [{
						xtype: 'container',
						ref: '../../titleBox',
						listeners: {
							scope: this,
							afterrender: this.afterRenderTitleHandler
						},
						items: {
							xtype: 'box',
							html: '<h1 class="sb-dashboard-tile-title">' + Sb.Locale.get('statistics') +
									' <span class="sb-dashboard-ga-logout">' + '(<a href="#">' + Sb.Locale.get('gaLogout') + '</a>)' + '</span></h1>'
						}
					}, {
						xtype: 'container',
						ref: '../../contentBox',
						cls: 'sb-dashboard-tile-content'
					}]
				}]
			}]
		});

		Sb.dashboard.tile.GoogleAnalyticsTile.superclass.initComponent.call(this);

		if (window.google && window.google.visualization) {
			this.model.loadSettings();
		}

		this.titleBox.height = 50;
		this.titleBox.add({
			xtype: 'label',
			text: this.model.startDate.format(Sb.Config.locales[Sb.Config.locale].dateFormat) + ' - ' +
					this.model.endDate.format(Sb.Config.locales[Sb.Config.locale].dateFormat)
		});
		this.showLoader();
	},

	afterRenderTitleHandler: function () {
		var a = this.getEl().child('.sb-dashboard-ga-logout a:first-child');
		this.toggleVisibleLogout();

		a.on('click', function () {
			this.model.toggleLoginState(false);
			this.toggleVisibleLogout();
			Sb.GoogleAnalyticsService.logout();
		}, this);
	},

	loginStateChangeHandler: function (model, isLogged) {
		this.toggleVisibleLogout();
		if (isLogged) {
			this.loadProfilesList();
		} else {
			this.renderLinkToAuthorize();
		}
	},

	settingsLoadHandler: function () {
		if (!Sb.Editor.model.get('googleAnalyticsCode')) {
			this.showError(Sb.Locale.get('gaCodeNotInstalled') + ' ', 'warning');
			this.appendShowEmbedCodeLink(this.getContentEl());
		} else {
			this.loadProfilesList();
		}
	},

	appendShowEmbedCodeLink: function (component) {
		if (component.rendered) {
			this.renderShowEmbedCodeLink(component.messageBox.getEl().last());
		} else {
			this.mon(component.messageBox, 'afterrender', function (el) {
				this.renderShowEmbedCodeLink(el.getEl().last());
			}, this);
		}
	},

	renderShowEmbedCodeLink: function (el) {
		new Ext.BoxComponent({
			renderTo: el,
			autoEl: {
				tag: 'a',
				href: '#',
				html: Sb.Locale.get('gaShowEmbedCodeWindow')
			},
			listeners: {
				scope: this,
				render: function (c) {
					c.getEl().on('click', function () {
						this.manager.getWindow().closeHandler();
						var mainTabs = Sb.Editor.Ui.mainWindow.tabs;
						mainTabs.setActiveTab(mainTabs.settingsPanel);
						mainTabs.settingsPanel.tabs.setActiveTab(1);
						mainTabs.settingsPanel.googleAnWinOpen();
					}, this);
				}
			}
		});
	},

	dataChangeHandler: function () {
		var el = this.getContentEl(),
			panel = new Ext.Panel();
		el.removeAll();
		el.add([panel, {
			cls: 'sb-dashboard-ga-seemore',
			html: '<a href="#">' + Sb.Locale.get('seeMoreChart') + '</a>'
		}]);
		el.doLayout();
		el.getEl().child('a').on('click', function () {
			var dashboard = Sb.Editor.windowManager.getManager(9).get('dashboardWindow');
			dashboard.tabs.setActiveTab(dashboard.tabs.googleAnalyticsTab);
		});

		this.renderAreaChart(panel.getId(), this.model.data);
	},

	/**
	 * Show auth window
	 */
	requestAuth: function (callback) {
		var openParams = 'width=888,height=444,toolbar=0,location=0,scrollbars=0,resizable=0,status=0';
		window.open(this.getAnalyticsUrl(), 'Auth', openParams);

		Sb.pushGoogleAuthToken = callback;
	},

	/**
	 * Authentication with Google and save a permanent token
	 */
	authenticate: function () {
		var tile = this;
		this.requestAuth(function (token) {
			Sb.GoogleAnalyticsService.processAuth(
				decodeURIComponent(token),
				function (result, response) {
					if (result.success) {
						tile.model.toggleLoginState(true);
					}
				}
			);
		});
	},

	toggleVisibleLogout: function () {
		if (this.getEl()) {
			this.getEl().child('.sb-dashboard-ga-logout').setVisible(this.model.isLogged);
		}
	},

	/**
	 * Show "Log in" button
	 */
	renderLinkToAuthorize: function () {
		this.contentBox.removeAll();
		this.contentBox.add({
			cls: 'sb-dashboard-ga-login',
			items: {
				xtype: 'button',
				text: Sb.Locale.get('gaLogin'),
				handler: this.authenticate.createDelegate(this)
			}
		});
		this.contentBox.doLayout();
	},

	/**
	 * Get url for auth scope
	 */
	getAnalyticsUrl: function () {
		var returnUrl = Sb.Config.originUrl + Sb.Config.baseUrl + '/js/externals/google/auth.html';
		return 'https://www.google.com/accounts/AuthSubRequest?session=1' +
				'&scope=' + encodeURIComponent('https://www.googleapis.com/auth/analytics.readonly') +
				'&next=' + encodeURIComponent(returnUrl);
	},

	loadProfilesList: function () {
		this.showLoader();
		Sb.GoogleAnalyticsService.getProfilesList(
			function (result, response) {
				if (result.success) {
					this.model.toggleLoginState(true);
					if (result.data.length) {
						var profilesData = [];
						Ext.each(result.data, function (item) {
							profilesData.push([item.tableId, item.tableId]);
						});

						this.profiles = profilesData;

						if (this.model.get('tableId')) {
							this.model.startTask('loadData', true);
							return;
						}

						if (result.data.length == 1) {
							this.model.set('tableId', result.data[0].tableId);
							this.model.saveSettings();
							this.model.startTask('loadData', true);
						} else {
							this.renderProfilesSelect(this.getContentEl(), this.profiles);
						}
					} else {
						this.showError(Sb.Locale.get('gaIncorrectCode') + ' ');
						this.appendShowEmbedCodeLink(this.getContentEl());
					}
				} else {
					this.model.toggleLoginState(false);
					this.renderLinkToAuthorize();
				}
			},
			this
		);
	},

	renderProfilesSelect: function (container, profiles, selected) {
		var store = new Ext.data.SimpleStore({
			fields: ['tableId', 'profileName'],
			data: profiles
		});

		var comboBox = new Ext.form.ComboBox({
			store: store,
			mode: 'local',
			width: 250,
			value: selected,
			valueField: 'tableId',
			displayField: 'profileName',
			emptyText: Sb.Locale.get('gaSelectProfileId'),
			listeners: {
				scope: this,
				select: function (combo, record, index) {
					this.showLoader();
					this.model.set('tableId', record.data.tableId);
					this.model.saveSettings();
					this.model.loadData();
				}
			}
		});

		container.removeAll();
		container.add([{
			html: Sb.Locale.get('gaProfile')
		}, comboBox]);
		container.doLayout();
	},

	showLoader: function () {
		var contentEl = this.getContentEl();
		contentEl.removeAll();
		contentEl.add({
			autoWidth: true,
			autoHeight: true,
			cls: 'sb-dashboard-loading',
			html: '<img class="sb-dashboard-loading-icon" src="' + Sb.Config.skinUrl+ '/images/loading.gif" alt="" />'
		});
		contentEl.doLayout();
	},

	renderAreaChart: function (idElem, sourceData) {
		var data = new google.visualization.DataTable();
		data.addColumn('string', Sb.Locale.get('gaDays'));
		data.addColumn('number', this.model.currentMainMetric.label);
		data.addRows(sourceData);

		var options = {
			width: 675,
			height: 180,
			pointSize: 3,
			legend: {
				position: 'none'
			},
			hAxis: {
				showTextEvery: 6,
				allowContainerBoundaryTextCufoff: true
			},
			vAxis: {
				viewWindow: {
					min: 0
				}
			},
			chartArea: {
				left: 35,
				top: 5,
				width: '95%',
				height: '85%'
			}
		};

		var chart = new google.visualization.AreaChart(
			document.getElementById(idElem)
		);
		chart.draw(data, options);
	},

	showMessage: function (message, type) {
		type = type || 'text';
		var el = this.getContentEl();
		el.removeAll();
		el.add({
			xtype: 'box',
			ref: 'messageBox',
			cls: 'sb-dashboard-analytics-message sb-dashboard-message sb-dashboard-' + type,
			html: '<i></i><span>' + message + '</span>'
		});
		el.doLayout();
	}

});