/**
 * @class Sb.editor.BaseFormPanel
 * @extends Ext.form.FormPanel
 */
Ext.define('Sb.editor.BaseFormPanel', {
	extend: 'Ext.form.FormPanel',

	footerCssClass: 'sb-action-footer',
	hasOkCancelButtons: true,

	// private
	initComponent: function () {
		if (this.hasOkCancelButtons) {
			this.buttons = this.buttons || [];
			this.buttons.push({
				text: Sb.Locale.get('ok'),
				handler: this.okHandler,
				scope: this
			}, {
				text: Sb.Locale.get('cancelButtonTitle'),
				handler: this.cancelHandler,
				scope: this
			});
		}

		Sb.editor.BaseFormPanel.superclass.initComponent.apply(this, arguments);
	},

	initEvents: function () {
		Sb.editor.BaseFormPanel.superclass.initEvents.call(this);
		this.on('activate', this.onActivate, this);
	},

	/**
	 * Activate handler
	 */
	onActivate: function () {
		Sb.editor.UiContext.setNewContext({
			yesCallback: this.uiContextYesHandler.createDelegate(this),
			isYesDeferred: true,
			noCallback: this.uiContextNoHandler.createDelegate(this)
		});
	},

	/**
	 * OK button handler.
	 *
	 * @param {Ext.Button} btn
	 */
	okHandler: function (btn) {
		var me = this,
			mask = Ext.getBody().mask();

		mask.addClass('sb-editor-mask');
		btn.setIconClass('sb-loading-icon');
		this.saveHandler({
			success: function() {
				btn.setIconClass('');
				Ext.getBody().unmask();
				Sb.editor.UiContext.setNewContext();
				me.closeDialog();
			},
			failure: function() {
				btn.setIconClass('');
				Ext.getBody().unmask();
			}
		});
	},

	/**
	 * UI context common handler for yes button.
	 */
	uiContextYesHandler: function () {
		this.saveHandler({
			success: Sb.editor.UiContext.finishDeferredAction.createDelegate(Sb.editor.UiContext),
			failure: Sb.editor.UiContext.cancelDeferredAction.createDelegate(Sb.editor.UiContext)
		});
	},

	/**
	 * Save handler. Must be overriden in child classes with calling in the end
	 * afterSaveCallback that arrive to it always.
	 */
	saveHandler: Ext.emptyFn,

	/**
	 * Get all active panel`s form errors.
	 *
	 * @return {Array}
	 */
	getActiveErrors: function () {
		var errorMsg = [];
		this.form.items.each(function (formItem) {
			if (formItem.isComposite) {
				formItem.eachItem(function (compositeChild) {
					if (compositeChild.activeError) {
						errorMsg.push(compositeChild.activeError);
					}
				});
			} else {
				if (formItem.activeError) {
					errorMsg.push(formItem.activeError);
				}
			}
		});
		return Ext.unique(errorMsg);
	},

	/**
	 * Shows all active panel`s form errors if they exists.
	 *
	 * @return {Boolean} Error show result
	 */
	showActiveErrors: function () {
		var errorMsg = this.getActiveErrors();
		Sb.Editor.Ui.mainWindow.statusBar.clear();
		if (errorMsg.length > 0) {
			Ext.each(errorMsg, function (msg) {
				Sb.Editor.Ui.mainWindow.statusBar.addMessageError(msg);
			});
			return true;
		}
		return false;
	},

	/**
	 * Cancel button handler.
	 */
	cancelHandler: function () {
		this.discardHandler();
		Sb.editor.UiContext.setNewContext();
		this.closeDialog();
	},

	/**
	 * UI context common handler for no button.
	 */
	uiContextNoHandler: function () {
		this.discardHandler();
	},

	/**
	 * Discard handler for undo and another actions.
	 */
	discardHandler: Ext.emptyFn,

	/**
	 * Close dialog by restoring tabs in it`s initial state, when all was hidden.
	 */
	closeDialog: function () {
		Sb.Editor.Ui.mainWindow.tabs.setActiveTab(null);
	}
});