/**
 * @class Sb.editor.MoveableTreeNodeUI
 * @extends Ext.tree.TreeNodeUI
 */
Ext.define('Sb.editor.MoveableTreeNodeUI', {
	extend: 'Ext.tree.TreeNodeUI',

	/**
	 * @cfg {String} elements
	 * A comma-delimited list of move controls to initialize when the node is rendered.
	 * Valid values are<div class="mdetail-params"><ul>
	 * <li><code>up</code></li>
	 * <li><code>right</code></li>
	 * <li><code>down</code></li>
	 * <li><code>left</code></li>
	 * </ul></div>
	 * Defaults to '<code>up,right,down,left</code>'.
	 */
	controls: 'up,right,down,left',

	// override parent implementation
	renderElements: function (n, a, targetNode, bulkRender) {
		this.indentMarkup = n.parentNode ? n.parentNode.ui.getChildIndent() : '';

		var cb = Ext.isBoolean(a.checked),
			nel,
			href = a.href ? a.href : Ext.isGecko ? '' : '#',
			buf = ['<li class="x-tree-node"><div ext:tree-node-id="', n.id, '" class="x-tree-node-el x-tree-node-leaf x-unselectable ', a.cls, '" unselectable="on">',
			'<div class="x-tree-node-el-inner">',
			'<span class="x-tree-node-indent">', this.indentMarkup, '</span>',
			'<img src="', this.emptyIcon, '" class="x-tree-ec-icon x-tree-elbow" />',
			'<img src="', a.icon || this.emptyIcon, '" class="x-tree-node-icon', (a.icon ? " x-tree-node-inline-icon" : ""), (a.iconCls ? " "+a.iconCls : ""), '" unselectable="on" />',
			cb ? ('<input class="x-tree-node-cb" type="checkbox" ' + (a.checked ? 'checked="checked" />' : '/>')) : '',
			'<a hidefocus="on" class="x-tree-node-anchor" href="', href, '" tabIndex="1" ',
			a.hrefTarget ? ' target="' + a.hrefTarget + '"' : '', '><span unselectable="on">',
			Ext.util.Format.htmlEncode(Ext.util.Format.ellipsis(n.text, 35 - n.getDepth() * 2)), '</span></a>',
			'<img src="', this.emptyIcon, '" class="x-tree-node-icon sb-hidden-icon" />',
			'<img src="', this.emptyIcon, '" class="x-tree-node-icon sb-protected-icon" />',
			'<div class="sb-pages-tree-node-controls"></div>',
			'</div></div>',
			'<ul class="x-tree-node-ct" style="display:none;"></ul>',
			'</li>'].join('');

		if (bulkRender !== true && n.nextSibling && (nel = n.nextSibling.ui.getEl())) {
			this.wrap = Ext.DomHelper.insertHtml("beforeBegin", nel, buf);
		} else {
			this.wrap = Ext.DomHelper.insertHtml("beforeEnd", targetNode, buf);
		}

		this.elNode = this.wrap.childNodes[0];
		this.ctNode = this.wrap.childNodes[1];
		var cs = this.elNode.firstChild.childNodes;
		this.indentNode = cs[0];
		this.ecNode = cs[1];
		this.iconNode = cs[2];
		var index = 3;
		if (cb) {
			this.checkbox = cs[3];
			index++;
		}
		this.anchor = cs[index];
		this.textNode = cs[index].firstChild;

		var iconsCount = 2; // hidden & protected
		this.controlsWrapper = Ext.get(cs[index + 1 + iconsCount]);
		var controls = this.controls;
		this.controls = {};
		this.newPos = {};
		if (controls === '') {
			this.controlsWrapper.dom.style.display = 'none';
		} else {
			Ext.each(controls.split(','), function (name) {
				if (name === '') {
					return true;
				}
				this.controls[name] = this.controlsWrapper.createChild({
					cls: 'sb-pages-tree-node-control-' + name
				});

				Ext.QuickTips.register({
					target: this.controls[name],
					text: Sb.Locale.get('movePageTooltip')
				});
				this.controls[name].on('click', this.onControlClick.createDelegate(this, [name]), this, {stopPropagation: true});
				this.controls[name].on('dblclick', function (e) {
					e.stopPropagation();
				});

				this.newPos[name] = null;
			}, this);
		}
	},

	/**
	 * Get drag-n-drop handles, that will allow drag of this node.
	 */
	getDDHandles: function () {
		return [this.ecNode, this.iconNode, this.textNode, this.elNode.firstChild];
	},

	/**
	 * Determine that we can move the current node in the specified node
	 * @param {Ext.tree.TreeNode} toNode
	 * @return {Boolean}
	 */
	allowMoveToNode: function (toNode) {
		if (!toNode) {
			return false;
		}
		if (!toNode.allowChildren) {
			return false;
		}
		if (!this.node.attributes.maxDepth) {
			return true;
		}
		var lastChildDepth = toNode.getDepth() + 1;
		var nodeDepth = this.node.getDepth();
		var childrenMaxDepth = 0;
		var computeDepth = function (nodeWithChildren) {
			Ext.each(nodeWithChildren.childNodes, function (childNode) {
				var difference = childNode.getDepth() - nodeDepth;
				if (difference > childrenMaxDepth) {
					childrenMaxDepth = difference;
				}
				computeDepth(childNode);
			}, this);
		};
		computeDepth(this.node);
		lastChildDepth += childrenMaxDepth;
		return lastChildDepth <= this.node.attributes.maxDepth;
	},

	onSelectedChange: function (state) {
		if (state) {
			this.addClass('x-tree-selected');
			this.node.getOwnerTree().body.scrollChildIntoView(this.controlsWrapper);
			this.focus();

			// check left direction
			if (this.controls.left) {
				if (this.node.getDepth() > 1) {
					this.newPos.left = {
						parentNode: this.node.parentNode.parentNode,
						refNode: this.node.parentNode.nextSibling
					};
				} else {
					this.newPos.left = null;
				}
			}

			// check up direction
			if (this.controls.up) {
				if (this.node.previousSibling) {
					this.newPos.up = {
						parentNode: this.node.previousSibling.parentNode,
						refNode: this.node.previousSibling
					};
				} else if (this.node.getDepth() > 1 && this.node.parentNode.previousSibling) {
					this.newPos.up = {
						parentNode: this.node.parentNode.previousSibling,
						refNode: null
					};
				} else {
					this.newPos.up = null;
				}
			}

			// check right direction
			if (this.controls.right) {
				if (this.allowMoveToNode(this.node.previousSibling)) {
					this.newPos.right = {
						parentNode: this.node.previousSibling,
						refNode: null
					};
				} else {
					this.newPos.right = null;
				}
			}

			// check down direction
			if (this.controls.down) {
				if (this.node.nextSibling) {
					this.newPos.down = {
						parentNode: this.node.nextSibling.parentNode,
						refNode: this.node.nextSibling.nextSibling
					};
				} else if (this.node.getDepth() > 1 && this.node.parentNode.nextSibling) {
					this.newPos.down = {
						parentNode: this.node.parentNode.nextSibling,
						refNode: this.node.parentNode.nextSibling.firstChild
					};
				} else {
					this.newPos.down = null;
				}
			}

			Ext.iterate(this.newPos, function (dir, pos) {
				this.controls[dir][(null === pos ? 'add' : 'remove') + 'Class']('sb-pages-tree-node-control-' + dir + '-disabled');
			}, this);
		} else {
			this.removeClass('x-tree-selected');
		}
	},

	onControlClick: function (name) {
		if (this.newPos[name]) {
			this.newPos[name].parentNode.expand();
			this.newPos[name].parentNode.insertBefore(this.node, this.newPos[name].refNode);
		}
	},

	focus: function () {
		if (this.node.parentNode) {
			Ext.get(this.node.parentNode.ui.ctNode).stopFx();
		}
		Sb.editor.MoveableTreeNodeUI.superclass.focus.apply(this, arguments);
	}
});
