/**
 * Page protect dialog for pages panel
 *
 * @class Sb.editor.PageProtectDialog
 * @extends Sb.Window
 */
Ext.define('Sb.editor.PageProtectDialog', {
	extend: 'Sb.Window',

	layout			: 'form',
	labelAlign		: 'top',
	width			: 400,
	title			: Sb.Locale.get('passwordProtectDialogTitle'),
	modal			: true,
	resizable		: false,
	cls				: 'sb-dialog-protection',
	login			: '',
	password		: '',
	defaultButton	: 0,
	okHandler		: Ext.emptyFn,
	cancelHandler	: Ext.emptyFn,
	closeAction		: 'onCancelClick',

	initComponent: function () {
		this.buttons = [{
			text		: Sb.Locale.get('ok'),
			handler		: this.onOkClick.createDelegate(this)
		}, {
			text		: Sb.Locale.get('cancelButtonTitle'),
			handler		: this.onCancelClick.createDelegate(this)
		}];

		var textFieldCfg = {
			anchor		: '100%',
			stripCharsRe: new RegExp('[^0-9a-z ' + Ext.escapeRe('!"#$%&\'()*+,-./;<=>?@[\\]^_`{|}~') + ']', 'gi'),
			allowBlank	: false,
			maxLength	: 255,
			msgTarget	: 'under'
		};

		/**
		 * Login field
		 * @type Ext.form.TextField
		 * @property loginField
		 */
		this.loginField = new Ext.form.TextField(Ext.applyIf({
			fieldLabel	: Sb.Locale.get('passwordProtectDialogLogin')
		}, textFieldCfg));
		if (this.login) {
			this.loginField.setValue(this.login);
		}

		/**
		 * Password field
		 * @type Ext.form.TextField
		 * @property passwordField
		 */
		this.passwordField = new Ext.form.TextField(Ext.applyIf({
			fieldLabel	: Sb.Locale.get('passwordProtectDialogPassword')
		}, textFieldCfg));
		if (this.password) {
			this.passwordField.setValue(this.password);
		}

		this.items = [
			this.loginField,
			this.passwordField
		];

		Sb.editor.PageProtectDialog.superclass.initComponent.call(this);
	},

	/**
	 * Close dialog if it`s valid
	 */
	onOkClick: function () {
		var isLoginValid	= this.loginField.isValid(),
			isPasswordValid	= this.passwordField.isValid(),
			isFormValid		= isLoginValid && isPasswordValid;

		if (isFormValid) {
			this.okHandler(this);
			this.close();
		}
	},

	/**
	 * Close dialog anyway
	 */
	onCancelClick: function () {
		this.hide();
		this.cancelHandler(this);
		this.close.defer(1, this);
	}
});