/**
 * @class Sb.editor.StyleColorPanel
 * @extends Ext.Panel
 * Style color editor panel
 * @constructor
 * @param {Object} config The config object
 * @xtype stylecolorpanel
 */
Ext.define('Sb.editor.StyleColorPanel', {
	extend: 'Ext.Panel',
	xtype: 'stylecolorpanel',

	initComponent: function () {
		Ext.apply(this, {
			itemId: 'colors',
			title: Sb.Locale.get('styleColorsTab'),
			helperMessage: Sb.Locale.get('styleColorsTabDescription'),
			autoScroll: true,
			cls: 'sb-editor-style-colors-panel',
			defaults: {border: false},
			items: [{
				layout: 'form',
				labelAlign: 'top',
				items: {
					ref: '../bgPlace',
					xtype: 'containercombobox',
					pageAreas: ['site', 'watermark', 'header', 'footer', 'content'],
					itemCls: 'sb-style-fill-target',
					name: 'bgPlace',
					listeners: {
						scope: this,
						beforeselect: this.onBgPlaceBeforeSelect,
						select: function (combo) {
							this.onBgPlaceSelect(combo.getValue());
						}
					}
				}
			}, {
				layout: 'column',
				defaults: {
					border: false,
					layout: 'form'
				},
				items: [{
					columnWidth: 0.55,
					labelAlign: 'right',
					labelWidth: 80,
					items: [{
						layout: 'form',
						labelAlign: 'right',
						labelWidth: 80,
						border: false,
						cls: 'sb-editor-style-background-panel',
						items: [{
							xtype: 'label',
							cls: 'sb-editor-panel-heading sb-editor-panel-background-heading',
							text: Sb.Locale.get('background')
						}, this.generateColorField({
							fieldLabel			: Sb.Locale.get('backgroundColorOrImage'),
							itemCls				: 'sb-editor-style-colors-font',
							id					: 'backgroundColor',
							name				: 'backgroundColor',
							allowBackground		: true,
							allowOpacity		: true,
							listeners			: {
								scope: this,
								backgroundchange: function (colormenu, background) {
									if (Sb.Editor.model.design.setBackground(this.getSelectedPlace(), background)) {
										Sb.editor.UiContext.registerChanges();
									}
								}
							}
						})]
					}, {
						xtype: 'label',
						cls: 'sb-editor-panel-heading sb-editor-panel-font-colors-heading',
						text: Sb.Locale.get('fontColors')
					}, {
						xtype: 'compositefield',
						fieldLabel: Sb.Locale.get('textColorLabel'),
						combineErrors: false,
						itemCls: 'sb-editor-style-colors-font',
						items: [this.generateColorField({
							id: 'textColor',
							name: 'textColor'
						})]
					}, {
						xtype: 'compositefield',
						fieldLabel: Sb.Locale.get('linkColorLabel'),
						combineErrors: false,
						itemCls: 'sb-editor-style-colors-font',
						items: [this.generateColorField({
							id: 'linkColor',
							name: 'linkColor'
						})]
					}, {
						xtype: 'compositefield',
						fieldLabel: Sb.Locale.get('h1ColorLabel'),
						combineErrors: false,
						itemCls: 'sb-editor-style-colors-font',
						items: [this.generateColorField({
							id: 'h1Color',
							name: 'h1Color'
						})]
					}, {
						xtype: 'compositefield',
						fieldLabel: Sb.Locale.get('h2ColorLabel'),
						combineErrors: false,
						itemCls: 'sb-editor-style-colors-font',
						items: [this.generateColorField({
							id: 'h2Color',
							name: 'h2Color'
						})]
					}]
				}, {
					columnWidth: 0.45,
					cls: 'sb-editor-style-colors-preview',
					labelAlign: 'top',
					items: [{
						ref: '../../colorsPreview',
						xtype: 'displayfield',
						name: 'colorsPreview',
						cls: 'x-form-text',
						style: {
							'overflow': 'hidden'
						},
						anchor: '0',
						height: 166,
						value:	'<div class="pageContentText">' +
									'<h1>' + Sb.Locale.get('previewHeading') + ' 1</h1>' +
									'<h2>' + Sb.Locale.get('previewHeading') + ' 2</h2>' +
									'<p>' + Sb.Locale.get('previewText') + '</p>' +
								'</div>',
						fieldLabel: Sb.Locale.get('preview')
					}]
				}]
			}]
		});

		Sb.editor.StyleColorPanel.superclass.initComponent.apply(this, arguments);

		this.on('activate', this.loadColorsTab, this, {single: true});
		this.on('activate', this.updateColorsTab, this);
	},

	onHeightChange: function () {
		var bodyHeight = Ext.getBody().getHeight(),
			mainWindowHeight = Sb.Editor.Ui.mainWindow.getHeight(),
			diffHeight = bodyHeight - mainWindowHeight;

		if (diffHeight < 0) {
			this.body.setHeight(this.body.getHeight() + diffHeight);
		}
	},

	loadColorsTab: function () {
		this.bgPlace.setValue(this.bgPlace.getValue() || 'content');
		this.onBgPlaceSelect();
	},

	updateColorsTab: function () {
		var bgPlace = this.getSelectedPlace() || 'content',
			design = Sb.Editor.model.design,
			colorCmp;

		// Font color fields
		if (this.isContainerContent()) {
			Ext.each(['text', 'link', 'h1', 'h2'], function (key){
				var modelKey = bgPlace + key.toUpperCaseFirst() + 'Color';
				colorCmp = Ext.getCmp(key + 'Color');
				colorCmp.setValue(design.getDecoration('fontColor', modelKey));
			}, this);
		}

		// Background color/image field
		colorCmp = Ext.getCmp('backgroundColor');
		colorCmp.setAllowBlank(bgPlace != 'site');
		colorCmp.setValue(design.getDecoration('backgroundColor', bgPlace + 'Color'));
		colorCmp.setBackgroundUploadParams(Sb.Config.baseUrl + '/Upload/Background/', {
			container: bgPlace
		});
		colorCmp.setAllowHeaders(bgPlace === 'site');
		colorCmp.setBackgroundValue(design.getBackground(bgPlace));
		colorCmp.menu.setVisibleOpacity(bgPlace != 'site');

		this.onUpdatePreview();
	},

	onBgPlaceBeforeSelect: function (combo) {
		if (this.stylePanel.showActiveErrors()) {
			combo.collapse();
			return false;
		}
		return true;
	},

	onBgPlaceSelect: function () {
		var visible = this.isContainerContent(),
			elements = Ext.select('.sb-editor-style-background-panel ~ *, .sb-editor-style-colors-preview');

		if (visible) {
			elements.setVisible(true);
		}

		elements.animate(
			{ opacity: { to: (visible ? 1 : 0) } },
			0.3,
			function () {
				if (!visible) {
					elements.setVisible(false, 'display');
				}
			}
		);
		this.updateColorsTab();
	},

	onColorsTabColorFieldChange: function (field, value) {
		if (!field.isValid()) {
			return;
		}

		var bgPlace = this.getSelectedPlace(),
			design = Sb.Editor.model.design,
			section = null,
			property= null;

		this.onUpdatePreview();
		switch (field.name) {
			case 'backgroundColor':
				section = 'backgroundColor';
				property = bgPlace + 'Color';
				break;
			case 'textColor':
				section = 'fontColor';
				property = bgPlace + 'TextColor';
				break;
			case 'linkColor':
				section = 'fontColor';
				property = bgPlace + 'LinkColor';
				break;
			case 'h1Color':
				section = 'fontColor';
				property = bgPlace + 'H1Color';
				break;
			case 'h2Color':
				section = 'fontColor';
				property = bgPlace + 'H2Color';
				break;
		}

		var newProperties = {};
		newProperties[section] = {};
		newProperties[section][property] = value;
		if (!design.unserialize(newProperties)) {
			return;
		}

		if (value != design.getDecoration(section, property)) {
			Sb.editor.UiContext.registerChanges();
		}
	},

	/**
	 * Generates common config for field
	 *
	 * @param {Object} config
	 */
	generateColorField: function (config) {
		config.listeners = Ext.apply({
			scope: this,
			change: function (field, value) {
				this.onColorsTabColorFieldChange(field, value);
			}
		}, config.listeners || {});

		return Ext.apply({
			xtype: 'colordropdown',
			width: 108,
			allowColorPalette: true,
			menuCls: 'sb-editor-style-colors-dropdown-menu'
		}, config);
	},

	/**
	 * Return true if selected place such as
	 * watermark, site background, content, header e.t.c. is container for widgets
	 * @return {Boolean}
	 */
	isContainerContent: function (bgPlace) {
		bgPlace = bgPlace || this.getSelectedPlace();
		return !(bgPlace == 'site' || bgPlace == 'watermark');
	},

	/**
	 * update preview area
	 */
	onUpdatePreview: function () {
		var fields = {
			textColor: {
				el: this.colorsPreview.el.child('p'),
				style: 'color',
				used: this.isContainerContent()
			},
			linkColor: {
				el: this.colorsPreview.el.child('p a'),
				style: 'color',
				used: this.isContainerContent()
			},
			h1Color: {
				el: this.colorsPreview.el.child('h1'),
				style: 'color',
				used: this.isContainerContent()
			},
			h2Color: {
				el: this.colorsPreview.el.child('h2'),
				style: 'color',
				used: this.isContainerContent()
			},
			backgroundColor: {
				el: this.colorsPreview.el,
				style: 'background-color',
				used: true
			}
		};
		Ext.iterate(fields, function (item, index) {
			var color, field;
			if (fields[item].used) {
				field = Ext.getCmp(item);
				if (!field.isValid()) {
					return;
				}
				color = field.value;
				if (color == 'transparent' && fields[item].style != 'background-color') {
					color = 'inherit';
				}
				color = Sb.Editor.model.design.getColor(color);
				fields[item].el.setStyle(fields[item].style, color);
			}
		}, this);
	},

	getSelectedPlace: function () {
		return Sb.Editor.model.isRtl() ? Sb.Utils.swapLeftAndRight(this.bgPlace.getValue()) : this.bgPlace.getValue();
	}
});