/**
 * Class for text component edit panel
 *
 * @class Sb.editor.component.TextEditPanel
 * @extends Sb.editor.component.BaseEditPanel
 */
Ext.define('Sb.editor.component.TextEditPanel', {
	extend: 'Sb.editor.component.BaseEditPanel',

	autoHeight: true,
	width: 500,

	initComponent: function () {
		var controlCls = 'sb-edit-control sb-edit-control-';
		this.fontFamilySizePanel = new Sb.form.FontFamilySizePanel({
			widgetControlCls: controlCls,
			fontFamilyWidth	: 144,
			fontSizeWidth	: 72
		});
		this.colorDecorationGroup = new Sb.form.ColorDecorationPanel({
			widgetControlCls: controlCls,
			colorComboWidth: 108,
			children: 'color,decoration'
		});
		this.strokeCheckBox = new Ext.form.Checkbox({
			ctCls: 'sb-widget-textcomponent-stroke',
			fieldLabel: Sb.Locale.get('stroke')
		});
		this.strokeControl = {
			layout: 'form',
			labelAlign: 'top',
			items: this.strokeCheckBox
		};

		Ext.apply(this, {
			layout		: 'column',
			cls			: 'sb-edit-component-panel-text',
			items		: [this.fontFamilySizePanel, this.colorDecorationGroup, this.strokeControl]
		});

		Sb.editor.component.TextEditPanel.superclass.initComponent.apply(this, arguments);
	},

	initEvents: function () {
		Sb.editor.component.TextEditPanel.superclass.initEvents.apply(this, arguments);

		var fontFamily = this.fontFamilySizePanel.getComponent('FontFamily');
		this.mon(fontFamily, 'select', function () {
			this.component.setData({fontFamily: fontFamily.value});
		}, this);

		var fontSize = this.fontFamilySizePanel.getComponent('FontSize');
		this.mon(fontSize, 'valid', function () {
			this.component.setData({fontSize: fontSize.getRawValue()});
		}, this);

		var fontColor = this.colorDecorationGroup.getComponent('Color');
		this.mon(fontColor, 'change', function () {
			this.component.setData({fontColor: fontColor.getValue()});
		}, this);

		Ext.each(this.colorDecorationGroup.decorations, function (decoration) {
			var decorationButton = this.colorDecorationGroup.getComponent('Decoration').getComponent(decoration);
			this.mon(decorationButton, 'toggle', function () {
				var data = {};
				data['font' + decoration] = decorationButton.pressed;
				this.component.setData(data);
			}, this);
		}, this);

		this.strokeCheckBox.on('check', function (field, checked) {
			this.component.setData({stroke: checked});
		}, this);
	},

	show: function (component) {
		Sb.editor.component.TextEditPanel.superclass.show.apply(this, arguments);
		var data = this.component.data;

		this.fontFamilySizePanel.getComponent('FontFamily').setValue(data.fontFamily);
		this.fontFamilySizePanel.getComponent('FontSize').setValue(data.fontSize);

		var fontColor = this.colorDecorationGroup.getComponent('Color');
		fontColor.setValue(data.fontColor);

		Ext.each(this.colorDecorationGroup.decorations, function (decoration) {
			var decorationButton = this.colorDecorationGroup.getComponent('Decoration').getComponent(decoration);
			decorationButton.toggle(1 == data['font' + decoration]);
		}, this);

		this.strokeCheckBox.setValue(data.stroke);
	}
});