/**
 * @class Sb.form.ColorPalette
 * @extends Ext.Component
 * @xtype colorpalette
 */
Ext.define('Sb.form.ColorPalette', {
	extend: 'Ext.BoxComponent',
	xtype: 'palette',
	cls: 'sb-color-palette',

	/**
	 * @cfg {String} value
	 * The initial color name to highlight like "color2-2"
	 */
	value: null,

	constructor: function (config) {
		config = Ext.apply({
			colors: []
		}, config);

		Sb.form.ColorPalette.superclass.constructor.call(this, config);
	},

	initComponent: function(){
		this.callParent();

		this.addEvents(
			/**
			 * @event select
			 * Fires when a color is selected
			 * @param {Sb.form.ColorPalette} this
			 * @param {String} color name
			 */
			'select'
		);

		if (this.handler){
			this.on('select', this.handler, this.scope, true);
		}
	},

	onRender: function(container, position) {
		this.callParent(arguments);

		var html = '',
			cssClass = ' sb-color-palette-item';

		Ext.each(this.colors, function(column) {
			html += '<ul>';
			Ext.each(column, function(color) {
				html += '<li class="palette-background-' + color + cssClass + '"></li>';
			});
			html += '</ul>';
		});

		this.el.update(html);

		this.mon(this.el, 'click', function(event, target){
			var colorName = target.className.match(/(color[^\s]*)/)[1];
			this.select(colorName);
		}, this, {delegate: 'li'});
	},

	afterRender: function() {
		var selectedColor;

		this.callParent();

		if (this.value) {
			selectedColor = this.value;
			this.value = null;
			this.select(selectedColor, true);
		}
	},

	/**
	 * Selects the specified color in the palette (fires the {@link #select} event)
	 * @param {String} colorName
	 * @param {Boolean} [suppressEvent] True to stop the select event from firing. Defaults to <tt>false</tt>.
	 */
	select: function(colorName, suppressEvent) {
		this.clear();

		if (this.rendered) {
			this.el.child('li.palette-background-' + colorName).addClass('active');
		}

		this.value = colorName;
		if (suppressEvent !== true) {
			this.fireEvent('select', this, colorName);
		}
	},

	/**
	 * Unselect active color in palette
	 */
	clear: function() {
		if (this.rendered) {
			if (this.value) {
				this.el.child('li.palette-background-' + this.value).removeClass('active');
			}
		}
	}
});