/**
 * @class Sb.form.Expander
 * @extends Ext.form.CompositeField
 * @constructor
 */
Ext.define('Sb.form.Expander', {
	extend: 'Ext.form.CompositeField',
	xtype: 'expander',
	mode: 'side', //side or corner
	cls: 'sb-expander',
	expanded: false,
	width: 124,
	allowDecimals: false,
	spinnerWidth: 52,
	minValue: 0,
	maxValue: 999,
	msgTarget: 'under',
	validateOnBlur: false,

	initComponent: function () {
		var items = [];

		this.fieldsNames = this.mode == 'side' ? ['top', 'right', 'bottom', 'left'] : ['tl', 'tr', 'br', 'bl'];

		this.mainSpinnerField = new Ext.ux.form.SpinnerField({
			allowBlank: false,
			width: this.spinnerWidth,
			minValue: this.minValue,
			maxValue: this.maxValue,
			allowDecimals: this.allowDecimals,
			wrapperCls: 'sb-expander-spinner',
			enableKeyEvents: true,
			validateOnBlur: false,
			listeners: {
				scope: this,
				keyup: this.onMainSpinnerChange,
				change: this.onMainSpinnerChange,
				focus: this.onFocus,
				blur: this.onBlur
			}
		});

		this.valuesButton = new Ext.Button({
			cls: 'sb-expander-values-button',
			width: 102,
			height: 24,
			tabIndex: -1,
			handler: this.expand,
			scope: this
		});

		this.buttonExpand = new Ext.Button({
			cls: 'sb-expander-expand-button',
			iconCls: 'sb-expander-expand-icon',
			width: 24,
			height: 24,
			tabIndex: -1,
			handler: this.expand,
			scope: this
		});

		this.buttonCollapse = new Ext.Button({
			cls: 'sb-expander-collapse-button',
			iconCls: 'sb-expander-collapse-icon',
			width: 24,
			height: 24,
			tabIndex: -1,
			handler: this.collapse,
			scope: this
		});

		items.push([this.mainSpinnerField, this.valuesButton, this.buttonExpand, this.buttonCollapse]);

		Ext.each(this.fieldsNames, function (name) {
			this[name + 'Field'] = new Ext.ux.form.SpinnerField({
				allowBlank: false,
				width: this.spinnerWidth,
				minValue: this.minValue,
				maxValue: this.maxValue,
				allowDecimals: this.allowDecimals,
				wrapperCls: 'sb-expander-' + name,
				enableKeyEvents: true,
				validateOnBlur: false,
				listeners: {
					scope: this,
					keyup: this.onFieldChange,
					change: this.onFieldChange,
					focus: this.onFocus,
					blur: this.onBlur
				}
			});
			items.push(this[name + 'Field']);
		}, this);

		Ext.apply(this, {items: items});

		this.cls += ' ' + this.mode;

		Sb.form.Expander.superclass.initComponent.apply(this, arguments);

		this.addEvents('expand', 'collapse');
	},

	afterRender: function () {
		Sb.form.Expander.superclass.afterRender.apply(this, arguments);

		if (this.expanded) {
			this.doExpand();
		} else {
			this.doCollapse();
		}
	},

	buildCombinedErrorMessage: function() {
		return Sb.Locale.get('expanderInvalidMessage', this.minValue, this.maxValue);
	},

	onFocus: function () {
		this.addClass('sb-expander-focus');
		this.fireEvent('focus', this);
	},

	onBlur: function () {
		this.removeClass('sb-expander-focus');
		this.fireEvent('blur', this);
	},

	onFieldChange: function (changedField) {
		var values = [],
			value = changedField.isValid(true) ? changedField.getValue() : 0,
			oldValue = this.value;

		Ext.each(this.fieldsNames, function (fieldName) {
			var field = this[fieldName + 'Field'],
				fieldValue = field === changedField ? value : field.getRawValue();

			values.push(fieldValue);
		}, this);

		this.value = this.valuesToCss(values);

		if (oldValue !== this.value && this.isValid(true)) {
			this.fireEvent('change', this, this.value, oldValue);
		}
	},

	onMainSpinnerChange: function (field) {
		var value = field.isValid(true) ? field.getValue() : 0,
			oldValue = this.value;
		this.value = this.valuesToCss(value);

		if (oldValue !== this.value && this.isValid(true)) {
			Ext.each(this.fieldsNames, function (fieldName) {
				var field = this[fieldName + 'Field'];
				field.setValue(value);
			}, this);

			this.fireEvent('change', this, this.value, oldValue);
		}
	},

	getValue : function(){
		return this.value;
	},

	setValue: function (cssValue) {
		if (cssValue == this.getValue() && !this.activeError) {
			return this;
		}

		var values = this.cssToValues(cssValue);

		Ext.each(this.fieldsNames, function (fieldName, index) {
			this[fieldName + 'Field'].setValue(values[index]);
			if (!this[fieldName + 'Field'].isValid(true)) {
				values[index] = 0;
			}
		}, this);
		this.updateCollapsedFieldsByValues(values);

		return Sb.form.Expander.superclass.setValue.apply(this, arguments);
	},

	/**
	 * Updates spinner and compact button values
	 * @param {Array} values Array with 4 values
	 */
	updateCollapsedFieldsByValues: function (values) {
		if (this.expanded) {
			return;
		}

		if (!values) {
			values = [];
			Ext.each(this.fieldsNames, function (fieldName) {
				var field = this[fieldName + 'Field'],
					fieldValue = field.getRawValue();
				values.push(fieldValue);
			}, this);
		}

		if (values[0] === values[1] && values[0] === values[2] && values[0] === values[3]) {
			if (this.rendered) {
				this.el.addClass('compact');
			}
			this.setWidth(86);
			this.mainSpinnerField.setValue(values[0]);
			return;
		}

		this.setWidth(126);
		this.mainSpinnerField.setValue(0);
		if (this.rendered) {
			this.el.removeClass('compact');
		}
		this.valuesButton.setText(values.join(','));
	},

	collapse: function () {
		if (!this.expanded) {
			return;
		}

		this.doCollapse();
	},

	doCollapse: function () {
		if (this.rendered) {
			this.el.removeClass('expanded');
			this.el.addClass('collapsed');
		}

		this.expanded = false;
		this.updateCollapsedFieldsByValues();
		this.fireEvent('collapse', this);
	},

	expand: function () {
		if (this.expanded) {
			return;
		}

		this.doExpand();
	},

	doExpand: function () {
		if (this.rendered) {
			this.el.removeClass('collapsed');
			this.el.addClass('expanded');
		}
		this.setWidth(138);
		this.expanded = true;
		this.fireEvent('expand', this);
	},

	isValid: function(preventMark) {
		var isValid = true;

		if (this.expanded) {
			Ext.each(this.fieldsNames, function (fieldName) {
				isValid = isValid && this[fieldName + 'Field'].isValid(preventMark);
				return isValid;
			}, this);
		} else {
			isValid = this.mainSpinnerField.isValid(preventMark);
		}

		return isValid;
	},

	/**
	 * @param {Array|Number|String} values Array with 4 values or one value
	 * @return {String} css rule
	 */
	valuesToCss: function (values) {
		var css = [],
			value;

		if (Ext.isArray(values)) {
			Ext.each(values, function (value) {
				css.push(value + 'px');
			});
		} else {
			value = values + 'px';
			css = [value, value, value, value];
		}

		return css.join(' ');
	},

	/**
	 * @param {String} css rule
	 * @return {Array} Array with 4 values
	 */
	cssToValues: function (css) {
		return css.replace(/px/g, '').split(' ');
	}
});
