/**
 * @class Sb.form.FontFamilyComboBox
 * @extends Ext.form.ComboBox
 * Control for selecting font family
 * @constructor
 * Create a new FontFamilyComboBox.
 * @param {Object} config Configuration options
 * @xtype sbfontfamilycombo
 */
Ext.define('Sb.form.FontFamilyComboBox', {
	extend: 'Ext.form.ComboBox',
	xtype: 'sbfontfamilycombo',

	triggerAction: 'all',
	mode: 'local',
	valueField: 'fontFamilyCss',
	displayField: 'value',
	focusAfterSelectEvent: true,
	editable: false,
	lazyInit: false,
	tpl: new Ext.XTemplate(
		'<tpl for=".">',
			'<div class="x-combo-list-item sb-font-{[values.value.replace(/ /g, \'-\').toLowerCase()]}"></div>',
		'</tpl>'
	),
	fieldLabel: Sb.Locale.get('fontFamily'),
	minListWidth: 150,

	initComponent: function () {
		if (!this.store) {
			var locale = Sb.Editor.siteView.getCurrentSiteLocale();

			if (locale == 'zh_CN' || locale == 'zh_TW') {
				this.store = Sb.FontManager.chineseFonts;
			} else if (locale == 'ja_JP') {
				this.store = Sb.FontManager.basicFonts;
			} else if (locale == 'ru_RU') {
				this.store = Sb.FontManager.cyrillicFonts;
			} else {
				this.store = Sb.FontManager.latinFonts;
			}
		}

		Sb.form.FontFamilyComboBox.superclass.initComponent.call(this);
	},

	onSelect: function (record) {
		Sb.form.FontFamilyComboBox.superclass.onSelect.apply(this, arguments);
		Sb.FontManager.loadFont(record.data.value);
	},

	afterRender: function () {
		Sb.form.FontFamilyComboBox.superclass.afterRender.apply(this, arguments);

		Ext.iterate(Sb.FontManager.fontFamilies, function (group, name) {
			var index = this.store.find('group', group);
			if (index == -1) {
				return true;
			}

			Ext.DomHelper.insertBefore(this.view.getNode(index), {
				cls: 'x-combo-list-group',
				html: name
			});
		}, this);
	}
});