/**
 * @class Sb.form.TreeComboBox
 * @extends Ext.form.ComboBox
 */
Ext.define('Sb.form.TreeComboBox', {
	extend: 'Ext.form.ComboBox',
	xtype: 'sbtreecombobox',

	tree: null,
	treeId: 0,

	initComponent: function () {
		this.treeId = Ext.id();
		this.focusLinkId = Ext.id();

		Ext.apply(this, {
			store: new Ext.data.SimpleStore({fields: [], data: [[]]}),
			editable: false,
			shadow: false,
			mode: 'local',
			triggerAction: 'all',
			maxHeight: 200,
			tpl: '<tpl for="."><div style="height:200px"><div id="' + this.treeId + '"></div><a href="#" class="x-grid3-focus" id="' + this.focusLinkId + '" tabIndex="-1"/></div></tpl>',
			selectedClass: '',
			onSelect: Ext.emptyFn,
			displayField: 'text',
			valueField: 'id'
		});

		var treeConfig = {
			border: false
		};

		Ext.apply(treeConfig, this.treeConfig);

		if (!treeConfig.root) {
			treeConfig.root = new Ext.tree.AsyncTreeNode({text: 'treeRoot', id: '0'}); 
		}

		this.tree = new Ext.tree.TreePanel(treeConfig);

		this.on('expand', this.onExpand);
		this.tree.on('click', this.onClick, this);

		Sb.form.TreeComboBox.superclass.initComponent.call(this);
	},

	setValue: function (v) {
		var text = v;
		if (this.valueField) {
			var node = this.findNode(this.valueField, v);
			if (node) {
				text = node[this.displayField];
			} else if(Ext.isDefined(this.valueNotFoundText)) {
				text = this.valueNotFoundText;
			} else {
				text = v = '';
			}
		}
		this.lastSelectionText = text;
		if (this.hiddenField) {
			this.hiddenField.value = Ext.value(v, '');
		}
		Ext.form.ComboBox.superclass.setValue.call(this, text);
		this.value = v;
		return this;
	},

	assertValue: function () {
		var val = this.getRawValue(),
			node;

		if (this.valueField && Ext.isDefined(this.value)) {
			node = this.findNode(this.valueField, this.value);
		}
		if (node && this.valueField) {
			if (this.value == val) {
				return;
			}
			val = node[this.valueField || this.displayField];
		}
		this.setValue(val);
	},

	findNode: function (prop, value) {
		var node;
		var root = this.tree.getRootNode();
		if (root[prop] == value) {
			node = root;
		} else {
			node = this.tree.getRootNode().findChild(prop, value, true);
		}
		return node;
	},

	onTriggerClick: function () {
		if (this.disabled) {
			return;
		}
		if (this.isExpanded()) {
			this.collapse();
		} else {
			this.onFocus();
			if (this.triggerAction == 'all') {
				this.doQuery(this.allQuery, true);
			} else {
				this.doQuery(this.getRawValue());
			}
		}
	},

	onFocus: function () {
		Sb.form.TreeComboBox.superclass.onFocus.call(this);
		Ext.get(this.focusLinkId).focus();
	},

	onClick: function (node) {
		if (this.fireEvent('beforeselect', this, node) !== false) {
			this.setValue(node[this.valueField]);
			this.collapse();
			this.fireEvent('select', this, node);
		}
	},

	onExpand: function () {
		this.tree.render(this.treeId);
		this.tree.focus();
		if (this.valueField && Ext.isDefined(this.value)) {
			var node = this.findNode(this.valueField, this.value);
			if (node) {
				node.select();
			} else {
				this.tree.getSelectionModel().clearSelections();
			}
		}
	},

	onViewClick : function (doFocus){
		//do nothing
	}
});