Ext.define('Sb.layout.Carousel', {
	extend: 'Ext.layout.ContainerLayout',

	scrollIncrement : 10,

	onLayout: function (ct, target) {
		if (!this.scrollEl) {
			this.scrollEl = target.createChild({
				cls: 'sb-carousel-layout',
				cn: [{
						cls: 'sb-carousel-left-scrollbutton'
					}, {
						cls: 'sb-carousel-right-scrollbutton'
					}, {
					cls: 'sb-carousel-scroller',
					cn: {
						cls: 'sb-carousel-body',
						cn: {
							cls: 'sb-carousel-wrapper'
						}
					}
				}]
			});
		}

		this.leftBtn = this.scrollEl.child('.sb-carousel-left-scrollbutton');
		this.rightBtn = this.scrollEl.child('.sb-carousel-right-scrollbutton');
		this.scroller = this.scrollEl.child('.sb-carousel-scroller');
		this.wrapper = this.scroller.child('.sb-carousel-wrapper');

		this.leftRepeater = new Ext.util.ClickRepeater(this.leftBtn, {
			interval: 10,
			delay: 0,
			handler: this.onScrollLeftClick,
			scope: this
		});
		this.rightRepeater = new Ext.util.ClickRepeater(this.rightBtn, {
			interval: 10,
			delay: 0,
			handler: this.onScrollRightClick,
			scope: this
		});

		this.renderAll(ct, this.wrapper);
		this.updateScrollButtons();
	},

	updateScrollButtons: function () {
		var pos = this.getScrollPos();
		this.leftBtn[pos === 0 ? 'addClass' : 'removeClass']('sb-carousel-button-disabled');
		this.rightBtn[pos === this.getMaxScrollPosition() ? 'addClass' : 'removeClass']('sb-carousel-button-disabled');
	},

	getNewScrollPosition: function (direction) {
		return direction == -1
			? this.getScrollPos() - this.scrollIncrement
			: this.getScrollPos() + this.scrollIncrement;
	},

	getScrollPos: function () {
		return this.scroller.dom.scrollLeft || 0;
	},

	getMaxScrollPosition: function () {
		return this.wrapper.getWidth() - this.scroller.getWidth();
	},

	onScrollLeftClick: function () {
		this.scrollTo(
			Math.max(0, this.getNewScrollPosition(-1))
		);
	},

	onScrollRightClick: function () {
		this.scrollTo(
			Math.min(this.getMaxScrollPosition(), this.getNewScrollPosition(1))
		);
	},

	scrollTo: function (position) {
		if (position === this.getScrollPos()) {
			return;
		}

		this.scroller.scrollTo('left', position);
		this.updateScrollButtons();
	}
});

Ext.Container.LAYOUTS.carousel = Sb.layout.Carousel;