/**
 * @class Sb.model.ContactWidget
 * @extends Sb.model.Widget
 * Class for contact widget model
 * @constructor
 * @param {Object} config The configuration options for the widget model
 */
Ext.define('Sb.model.ContactWidget', {
	extend: 'Sb.model.Widget',
	translatableFields: ['subject', 'reply', 'buttonText'],

	constructor: function (config) {
		config = Ext.apply({
			type: 'contact'
		}, config);

		this.fields = [];

		Sb.model.ContactWidget.superclass.constructor.call(this, config);

		this.addEvents('changefields');
	},

	changeLocale: function (newLocale) {
		var oldLocale = this.localeCode || Sb.Editor.model.get('locale');
		this.callParent([newLocale]);

		Ext.each(this.fields, function (field, index) {
			this.fields[index] = this.switchTranslations(field, newLocale, oldLocale, ['title']);
		}, this);

		this.fireEvent('changefields', this);
	},

	normalizeData: function (data) {
		var oldLocale = this.localeCode || Sb.Editor.model.get('locale');
		data = this.callParent([data]);
		Ext.each(data.fields, function (field, index) {
			data.fields[index] = this.switchTranslations(field, Sb.Editor.model.get('locale'), oldLocale, ['title']);
		}, this);

		return data;
	},

	updateTranslationList: function (locales) {
		this.callParent([locales]);

		Ext.each(this.fields, function (field) {
			this.clearTranslationList(field.translations, locales);
		}, this);
	},

	indexOf: function (name) {
		var index = -1;
		Ext.each(this.fields, function (field, i) {
			if (field.name == name) {
				index = i;
				return false;
			}
		});
		return index;
	},

	addField: function (attrs) {
		this.fields.push(attrs);
		this.fireEvent('change', this);
	},

	deleteField: function (name) {
		var index = this.indexOf(name);
		if (index == -1) {
			return;
		}

		this.fields.splice(index, 1);
		this.fireEvent('change', this);
	},

	insertField: function (name, newIndex) {
		var index = this.indexOf(name);
		if (index == -1 || index == newIndex) {
			return;
		}

		var field = this.fields[index];
		this.fields.splice(index, 1);
		this.fields.splice(newIndex, 0, field);
		this.fireEvent('change', this);
	},

	updateField: function (name, attrs) {
		var index = this.indexOf(name);
		if (index == -1) {
			return;
		}

		var field = this.fields[index];
		Ext.apply(field, attrs);
		this.fireEvent('change', this);
	},

	unserialize: function (data) {
		var isChanged = false;

		if (Ext.isArray(data.fields)) {
			this.fields = [];
			Ext.each(data.fields, function (field) {
				this.fields.push(Ext.apply({}, field));
			}, this);
			delete data.fields;
			isChanged = true;
		}

		var isDataChanged = Sb.model.ContactWidget.superclass.unserialize.call(this, data);
		if (!isDataChanged && isChanged) {
			this.fireEvent('change', this);
		}

		return isDataChanged || isChanged;
	},

	// :TODO: asinko 20111018: Need rework all logic about serialize/rootData/widgetList
	// because it is difficult to understand, has a sophisticated interface,
	// does not lend normal inheritance, uses extra memory where it is not necessary
	serialize: function (rootData) {
		var isInitiator	= !rootData; rootData = rootData || {};

		var data = Sb.model.ContactWidget.superclass.serialize.call(this);
		delete data.widgets;
		data.fields = [];
		Ext.each(this.fields, function (field) {
			data.fields.push(Ext.apply({}, field));
		});

		if (!rootData.widgetList) {
			rootData.widgetList = {};
		}
		if (!rootData.widgetList[this.data.uuid]) {
			rootData.widgetList[this.data.uuid] = data;
		}

		return (isInitiator) ? data : this.data.uuid;
	},

	onUnserializePropertyChange: function (key) {
		if (key === 'recipients') {
			this.updateIsConfiguredProperties();
		}
	},

	computeIsConfiguredProperties: function () {
		return !!this.get('recipients');
	}
});
