/**
 * @class Sb.model.HeaderWidget
 * @extends Sb.model.Widget
 * Class for header model
 * @constructor
 * @param {Object} config The configuration options for the widget model
 */
Ext.define('Sb.model.HeaderWidget', {
	extend: 'Sb.model.Widget',

	/**
	 * @cfg {String} type
	 * defaults to <tt>Header</tt>
	 */

	constructor: function (config) {
		config = Ext.apply({
			type: 'header'
		}, config);

		this.addEvents(
			/**
			 * @event heightchange
			 * Fires after height has changed.
			 */
			'heightchange',
			/**
			 * @event imagechange
			 * Fires after the image and it's properties has changed.
			 */
			'imagechange',
			/**
			 * @event imagefitchange
			 * Fires after the image fit has changed.
			 */
			'imagefitchange',
			/**
			 * @event logochange
			 * Fires after the logo and it's properties has changed.
			 */
			'logochange',
			/**
			 * @event titlechange
			 * Fires after the title and it's properties has changed.
			 */
			'titlechange',
			/**
			 * @event subtitlechange
			 * Fires after the subtitle and it's properties has changed.
			 */
			'subtitlechange',
			/**
			 * @event textchange
			 * Fires after the text and it's properties has changed.
			 */
			'textchange'
		);
		Sb.model.HeaderWidget.superclass.constructor.call(this, config);
	},

	/**
	 * Fires on changing of single property change to allow us fire up header widget change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: function (key) {
		var matches = key.match(/^(height|image|logo|title|subtitle|text)/);
		if ('imageFit' == key) {
			this.fireEvent('imagefitchange');
		} else if (matches) {
			this.fireEvent(matches[0] + 'change');
		}
	},

	getResourcesUrl: function () {
		return Sb.Editor.model.design.getResourcesUrl();
	},

	statics: {
		/**
		 * @param {string} imageName
		 * @returns {string}
		 */
		buildImagePath: function (imageName) {
			return Sb.Config.path.attachmentsFolder + '/' + Sb.Config.path.headerFolder + '/' + imageName;
		}
	}
});
