/**
 * @class Sb.model.TextWidget
 * @extends Sb.model.Widget
 * Class for text widget model
 */
Ext.define('Sb.model.TextWidget', {
	extend: 'Sb.model.Widget',

	type: 'text',
	translatableFields: ['content'],

	/**
	 * Return array of all anchors in content of text widget
	 *
	 * @return {Array} array of anchors like [{name, widgetId},..]
	 */
	collectAnchors: function () {
		var html = this.get('content'),
			anchors = [],
			reAnchors = new RegExp('<a[^>]+name=[\'\"]?([a-z]+[a-z0-9-_]*)[\'\"]?', 'gi'),
			array;

		if (html === '') {
			return anchors;
		}

		while ((array = reAnchors.exec(html)) != null) {
			anchors.push({
				name: array[1],
				widgetId: this.get('uuid')
			});
		}

		return anchors;
	},

	/**
	 * Is widget has content, that could be rendered? Used in renderPlaceholder, for instance.
	 *
	 * @returns {Boolean}
	 */
	isEmpty: function () {
		return !this.get('content');
	},

	unserialize: function (data) {
		if (data.reInitUuid) {
			data.content = data.content.replace(/page:\/\/[^"]+/g, 'page://' + Sb.Editor.model.rootPage.get('identity'));
		}

		return Sb.model.TextWidget.superclass.unserialize.call(this, data);
	}
});