/**
 * Class for Amazon Store Edit Panel
 *
 * @class Sb.widget.AmazonStoreEditPanel
 * @extends Sb.widget.WidgetEditPanel
 */
Ext.define('Sb.widget.AmazonStoreEditPanel', {
	extend: 'Sb.widget.WidgetEditPanel',

	initComponent: function () {
		var heightFieldUnit,
			heightPanel;

		this.widgetLabel = new Ext.form.Label();

		this.urlField = new Ext.form.TextField({
			ctCls: 'sb-widget-amazon-store-url-field',
			maxLength: 60,
			width: 545,
			hideLabel: true,
			regex: /^https?:\/\/astore\.amazon\.[a-z]+\/.+$/,
			msgTarget: 'under',
			regexText: Sb.WidgetTypeManager.get('amazon_store').getTranslation('invalidUrl')
		});

		this.heightField = new Ext.form.TextField({
			width: 40,
			allowNegative: false,
			decimalPrecision: 0,
			minValue: 100,
			minText: Sb.Locale.get('layoutColumnWidthMoreThan'),
			maxLength: 4,
			maxValue: 5000,
			msgTarget: 'under'
		});

		heightFieldUnit = new Ext.BoxComponent({
			autoEl: {
				tag: 'span'
			},
			cls: 'sb-layout-unit',
			html: Sb.Locale.get('px')
		});

		heightPanel = new Ext.Panel({
			fieldLabel: Sb.WidgetTypeManager.get('amazon_store').getTranslation('heightFieldLabel'),
			ctCls: 'sb-widget-amazon-store-height-panel',
			items: [
				this.heightField,
				heightFieldUnit
			]
		});

		Ext.applyIf(this, {
			layout: 'form',
			labelAlign: 'top',
			autoHeight: true,
			itemCls: 'sb-widget-edit-panel-amazon-store-form',
			cls: 'sb-widget-edit-panel-amazon-store',
			items: [
				this.widgetLabel,
				this.urlField,
				heightPanel
			]
		});
		Sb.widget.AmazonStoreEditPanel.superclass.initComponent.apply(this, arguments);
	},

	show: function (widget) {
		Sb.widget.AmazonStoreEditPanel.superclass.show.call(this, widget);

		this.bindModel({
			url: 'urlField',
			height: 'heightField'
		});

		this.urlField.setRawValue(widget.model.get('url'));
		this.heightField.setRawValue(widget.model.get('height'));
		this.widgetLabel.setText(widget.getType().getTranslation('codeLabel') + ' <a href="' +
			Sb.Editor.getHelpUrl('Widgets/' + widget.xtype.toUpperCaseFirst()) +
			'" target="_blank">' + Sb.Locale.get('learnMore') + '</a>', false);
	}
});