/**
 * @class Sb.widget.BlogPost
 * @extends Sb.widget.Text
 * Class for Blog Post Widget view
 * @xtype blog_post_widget
 */
Ext.define('Sb.widget.BlogPost', {
	extend: 'Sb.widget.Widget',
	xtype: 'blog_post_widget',

	cls: 'sb-widget-blog-post',
	renderMode: 'full',
	movable: false,
	translatable: true,

	initComponent: function () {
		if (this.renderMode === 'teaser') {
			this.hasEditButton = false;
			this.clickable = false;
		}
		Sb.widget.BlogPost.superclass.initComponent.apply(this, arguments);

		this.removalConfirmationMessage = Sb.Locale.get('widgetBlogPostConfirmRemoval');
	},

	onRender: function () {
		var items = [];

		Sb.widget.BlogPost.superclass.onRender.apply(this, arguments);

		this.postCreatedEl = this.widgetContentEl.createChild({cls: 'widget-blog_post-created'});
		this.postContentEl = this.widgetContentEl.createChild({cls: 'widget-blog_post-content'});
		this.postLinksEl = this.widgetContentEl.createChild({cls: 'widget-blog_post-links'});

		this.createdComponent = new Sb.editor.component.Date({
			renderTo	: this.postCreatedEl,
			parent		: this,
			hasControls	: false,
			movable		: false,
			overable	: this.renderMode == 'full',
			clickable	: this.renderMode == 'full',
			renderer	: function (value) {
				return Sb.Locale.get(
					'widgetBlogPostCreatedText',
					'<em>' + Ext.util.Format.date(value,
						Sb.Config.locales[Sb.Editor.siteView.getCurrentSiteLocale()].dateFormat
					) + '</em>'
				);
			},
			listeners: {
				scope: this,
				completeedit: function (cmp) {
					var date = cmp.getValue();
					this.model.set('createdTimestamp',
						Math.round(date.getTime() / 1000) - date.getTimezoneOffset() * 60);
					this.model.save();
				}
			}

		});

		switch (this.renderMode) {
			case 'full':
				this.contentContainerWidget = new Sb.widget.Container({
					renderTo	: this.postContentEl,
					model		: this.model
				});

				this.createWidgetTitle({
					emptyText: Sb.Locale.get('newBlogPostPage')
				});

				var blog = this.model.ownerPage.parentPage.widget.queryBy(function (widget) {
					return widget instanceof Sb.model.BlogWidget;
				})[0];
				var posts = Sb.Editor.getController('Blogs').getPosts(blog);
				var pos = posts.indexOf(this.model);
				var nextPost = posts.itemAt(pos - 1);
				var prevPost = posts.itemAt(pos + 1);

				if (prevPost || nextPost) {
					var linksEl = this.postLinksEl.createChild();
					if (prevPost) {
						linksEl.createChild({
							tag: 'a',
							href: '#',
							html: Sb.Locale.get('widgetBlogPreviousPostText')
						}).on('click', function () {
							Sb.Editor.siteView.setCurrentPage(prevPost.ownerPage);
						}, this, {stopEvent: true});
					}
					if (nextPost) {
						if (prevPost) {
							linksEl.createChild({tag: 'span', html: ' | '});
						}

						linksEl.createChild({
							tag: 'a',
							href: '#',
							html: Sb.Locale.get('widgetBlogNextPostText')
						}).on('click', function () {
							Sb.Editor.siteView.setCurrentPage(nextPost.ownerPage);
						}, this, {stopEvent: true});
					}
				}

				this.postLinksEl.createChild({
					cn: {
						tag		: 'a',
						href	: '#',
						html	: Sb.Locale.get('widgetBlogPostReturnToBlogText')
					}
				}).first().on('click', function () {
					Sb.Editor.siteView.setCurrentPage(blog.ownerPage);
				}, this, {stopEvent: true});

				this.mon(this.contentElement, 'contextmenu', Sb.Editor.Ui.onContextMenu, Sb.Editor.Ui, {
					type	: 'widget',
					widget	: this
				});

				break;

			case 'teaser':
				this.widgetTitle = this.widgetEl.createChild({
					tag: 'h2',
					cls: 'widget-title',
					cn: {
						tag: 'a',
						href: '#'
					}
				}, this.widgetContentEl);
				this.widgetTitle.child('a').on('click', function () {
					Sb.Editor.siteView.setCurrentPage(this.model.ownerPage);
				}, this, {stopEvent: true});

				this.model.widgets.each(function (widget) {
					if (widget.get('type') == 'blog_post_cut') {
						return false;
					}
					items.push({
						xtype : widget.data.type + '_widget',
						model : widget,
						overable: false,
						clickable: false
					});
				}, this);

				this.contentContainerWidget = new Ext.Container({
					renderTo	: this.postContentEl,
					items		: items
				});

				if (this.model.widgets.getCount() - 1 > items.length) {
					this.postLinksEl.createChild({
						cls: 'widget-blog_post-links-readmore',
						cn: {
							tag		: 'a',
							href	: '#',
							html	: Sb.Locale.get('widgetBlogPostReadMoreText')
						}
					}).child('a').on('click', function () {
						Sb.Editor.siteView.setCurrentPage(this.model.ownerPage);
					}, this, {stopEvent: true});
				}
				break;
		}
	},

	afterRender: function () {
		Sb.widget.BlogPost.superclass.afterRender.call(this);
		this.onWidgetTitleChange();
		this.refreshCreated();
	},

	initEvents: function () {
		Sb.widget.BlogPost.superclass.initEvents.call(this);

		this.mon(this.model.ownerPage, 'change', this.onWidgetTitleChange, this);
		this.mon(this.model.ownerPage, 'save', function () {
			Sb.Editor.Ui.mainWindow.statusBar.setMessage(Sb.Locale.get('messageChangesSaved'));
		});
		this.mon(this.model, 'propertychange', function (key, value, oldValue) {
			if (key == 'createdTimestamp') {
				this.refreshCreated();
			}
		}, this);
	},

	getWidgetTitle: function () {
		return this.model.ownerPage.get('title') || '';
	},

	setWidgetTitle: function (newTitle) {
		newTitle = (newTitle === '') ? Sb.Locale.get('newBlogPostPage') : newTitle;
		var postPage = this.model.ownerPage;
		if (this.getWidgetTitle() != newTitle) {
			postPage.set('title', newTitle);
			if (postPage.get('slugIsAuto') != '1') {
				postPage.save();
				return;
			}

			Sb.PageService.createSlug(newTitle, function (slug) {
				var re = new RegExp('^' + Ext.escapeRe(slug) + '(?:-(\\d+))?$', 'ig');
				if (re.test(postPage.get('slug'))) {
					postPage.save();
					return;
				}

				if (postPage.set('slug', slug)) {
					postPage.makeSlugUnique();
					postPage.save();
				}
			}, this);
		}
	},

	onWidgetTitleChange: function () {
		if (this.renderMode == 'teaser') {
			this.widgetTitle.child('a').update(this.getWidgetTitle());
			return;
		}

		Sb.widget.BlogPost.superclass.onWidgetTitleChange.call(this);
	},

	refreshCreated: function () {
		var date = new Date();
		date.setTime((parseInt(this.model.get('createdTimestamp'), 10) + date.getTimezoneOffset() * 60) * 1000);
		this.createdComponent.setValue(date);
	},

	removeComponentHandler: function () {
		Sb.widget.Widget.superclass.removeComponentHandler.call(this);
		this.model.ownerPage.parentPage.remove(this.model.ownerPage);
		this.model.ownerPage.parentPage.save();
		Sb.Editor.siteView.setCurrentPage(this.model.ownerPage.parentPage);
	},

	onStartEdit: function () {
		Sb.widget.BlogPost.superclass.onStartEdit.call(this);
		this.wasCommenting = !!Sb.Editor.getController('Blogs').findCommentingOnPost(this.model);
		this.pageMemento = this.model.ownerPage.saveToMemento();
	},

	onCancelEdit: function () {
		var fn = this.wasCommenting ? 'addCommentingToPost' : 'removeCommentingFromPost';
		Sb.Editor.getController('Blogs')[fn](this.model);
		// Delete widgets list because it can't be restored.
		// :TODO: agalitsyn 20130527: Remove after implementation of updating page's widget list during save process.
		delete this.pageMemento.widgetList;
		delete this.memento.widgetList;
		delete this.memento.widgets;
		this.model.ownerPage.restoreFromMemento(this.pageMemento);
		Sb.widget.BlogPost.superclass.onCancelEdit.call(this);
	},

	onCompleteEdit: function () {
		this.model.ownerPage.saveIfChanged(this.pageMemento);
		Sb.widget.BlogPost.superclass.onCompleteEdit.call(this);
	},

	onDestroy: function () {
		this.contentContainerWidget.destroy();
		Sb.widget.BlogPost.superclass.onDestroy.call(this);
	},

	onStylesChange: function (styles) {
		if (this.renderMode != 'teaser') {
			this.callParent([styles]);
		}
	},

	onCurrentLocaleChange: function (newLocale, oldLocale) {
		this.callParent([newLocale, oldLocale]);
		this.model.ownerPage.changeLocale(newLocale);
	}
});
