/**
 * @class Sb.widget.CommentingEditPanel
 * @extends Sb.widget.WidgetEditPanel
 */
Ext.define('Sb.widget.CommentingEditPanel', {
	extend: 'Sb.widget.WidgetEditPanel',

	defaultPosition: 't',
	disqusRegisterUrl: 'http://' + Sb.Config.disqusDomain + '/admin/register/',

	initComponent: function () {
		Ext.apply(this, {
			autoHeight: true,
			cls: 'sb-widget-edit-panel-commenting',
			items: [{
				id: 'commentingRegisterBlock',
				hidden: true,
				items: [{
					xtype: 'box',
					html: Sb.Locale.get('commentingWidgetRegisterTitle')
				}, {
					cls: 'sb-widget-edit-panel-commenting-register-button-block',
					items: [{
						xtype: 'button',
						text: Sb.Locale.get('register'),
						width: 70,
						cls: 'sb-widget-edit-panel-commenting-register',
						listeners	: {
							scope: this,
							click: function () {
								if (!window.open(this.disqusRegisterUrl, '_blank')) {
									Sb.Editor.Ui.mainWindow.statusBar.setMessageError(Sb.Locale.get('popupBlocked'));
								}
							}
						}
					}]
				}]
			}, {
				xtype: 'box',
				html: Sb.Locale.get('commentingWidgetShortNameTitle'),
				cls: 'sb-widget-edit-panel-shortname-title'
			}, {
				layout: 'column',
				items: [{
					width: 305,
					layout: 'form',
					items: [{
						id: 'commentingShortnameField',
						xtype: 'textfield',
						hideLabel: true,
						maxLength: 255,
						anchor: '100%',
						msgTarget: 'commentingShortnameFieldMsgTarget',
						allowBlank: false,
						regex: /^[a-z0-9\-_]*$/i,
						maskRe: /[a-z0-9\-_]/i,
						itemCls: 'sb-commenting-shortname-control',
						validationEvent: false
					}]
				}, {
					xtype: 'button',
					width: 70,
					cls: 'sb-widget-edit-panel-commenting-ok-button',
					text: Sb.Locale.get('apply'),
					listeners	: {
						scope: this,
						click: function (btn, evt) {
							var field = Ext.getCmp('commentingShortnameField'),
								value = field.getValue();
							field.clearInvalid();
							if (!field.isValid() || value == Sb.Editor.model.get('disqusId')) {
								return;
							}
							this.validateForum(value);
						}
					}
				}]
			}, {
				id: 'commentingShortnameFieldMsgTarget',
				xtype: 'box',
				cls: 'x-form-invalid-msg',
				autoWidth: true,
				style: {
					display: 'none'
				}
			}, {
				id: 'commentingShortnameDescription',
				xtype: 'box',
				cls: 'sb-widget-edit-panel-commenting-description',
				html: Sb.Locale.get('commentingWidgetShortNameDescription', this.disqusRegisterUrl)
			}]
		});

		Sb.widget.CommentingEditPanel.superclass.initComponent.apply(this, arguments);
	},

	show: function (widget) {
		Sb.widget.CommentingEditPanel.superclass.show.call(this, widget);
		if (widget.model.isConfiguredProperties()) {
			Ext.getCmp('commentingShortnameField').setValue(Sb.Editor.model.get('disqusId'));
		} else {
			Ext.getCmp('commentingRegisterBlock').show();
			Ext.getCmp('commentingShortnameField').setRawValue('');
		}
	},

	/**
	 * Check if forum was registered on Disqus
	 */
	validateForum: function (forum) {
		Sb.DisqusService.isForumExists(forum, function (result, response) {
			if (!result.success) {
				Sb.Editor.Ui.mainWindow.statusBar.setMessageError(Sb.Locale.get('commentingWidgetCreateError'));
				return;
			}
			if (result.data.value) {
				Ext.getCmp('commentingRegisterBlock').hide();
				this.widget.changeForum(forum);
			} else {
				Ext.getCmp('commentingShortnameField').markInvalid(Sb.Locale.get('commentingWidgetInvalidForumError'));
			}
		}, this);
	}
});
