/**
 * @class Sb.widget.Language
 * @extends Sb.widget.Widget
 * Class for language switcher widget
 * @xtype hidden_widget
 */
Ext.define('Sb.widget.Language', {
	extend: 'Sb.widget.Widget',
	xtype: 'language_widget',

	cls: 'sb-widget-language',

	initEvents: function () {
		this.mon(Sb.Editor.model, 'propertychange', function(key) {
			if (key === 'locales') {
				this.refreshContent();
			}
		}, this);
		this.mon(this.model, 'propertychange', function (key) {
			if (key === 'appearance') {
				this.refreshContent();
			}
		}, this);

		this.callParent();
	},

	renderContent: function () {
		Sb.ResourceManager.require([
			Sb.Config.siteResourcesUrl + '/components/' + Sb.Config.jsFrameworkFile
		], function () {
			Sb.ResourceManager.require([
				Sb.Utils.antiCache(
					Sb.Config.siteResourcesUrl + '/components/jquery/ms-dropdown/jquery.dd' +
						(Sb.Config.isDebug ? '' : '.min') + '.js',
					false
				),
				Sb.Utils.antiCache(
					Sb.Config.siteResourcesUrl + '/components/jquery/ms-dropdown/css/dd.css',
					false
				),
				Sb.Utils.antiCache(
					Sb.Config.siteResourcesUrl + '/modules/language/language.js',
					false
				),
				Sb.Utils.antiCache(
					Sb.Config.siteResourcesUrl + '/modules/language/language.css',
					false
				)
			], function () {
				var content = '', template = Sb.Editor.model.get('template').partials[this.getType().getPhpType()];
				if (Ext.isDefined(template)) {
					var partialTemplate = twig({data: template});
					content = partialTemplate.render(this.prepareData());
				}
				this.getContentTarget().update(content, true);
			}, this);
		}, this);
	},

	onContentElementClick: function (event) {
		var target, m;

		target = event.getTarget('a', 2);
		if (target) {
			this.changeLocale(target.getAttribute('href'));
		}

		this.callParent(arguments);
	},

	changeLocale: function (localeUrl) {
		var m = localeUrl.match('.*#(.*)');
		if (!m) {
			return;
		}
		var locale = m[1];
		if (Sb.Editor.model.get('locales').indexOf(locale) == -1) {
			return;
		}
		if (Sb.Editor.siteView.getCurrentSiteLocale() == locale) {
			return;
		}

		Sb.model.apiProvider.runAfterLastCall(function() {
			Sb.Loading.updateMessage(Sb.Locale.get('loaderMsgChangingSiteLanguage'));
			Sb.Loading.show();
			Ext.util.Cookies.set('currentSiteLocale', locale);
			window.location.reload();
		});
	},

	prepareData: function () {
		var locales = [],
			data = Sb.widget.Language.superclass.prepareData.call(this);
		Ext.each(Sb.Editor.model.get('locales'), function (locale) {
			locales.push({
				name: Sb.Config.locales[locale].name,
				url: '#' + locale,
				code: locale.split('_')[1].toLowerCase(),
				selected: locale == Sb.Editor.siteView.getCurrentSiteLocale() ? 'true' : 'false'
			});
		});
		data.framework = 'jQuery';
		data.baseUrl = Sb.Config.siteResourcesUrl + '/';
		data.locales = locales;
		data.onAfterRender = 'function() {' +
			'var widgetComponent = Ext.getCmp("' + this.getId() + '");' +
			'if (widgetComponent.getShowEditPanelState() && widgetComponent.getEditPanel().isVisible()) {' +
				'widgetComponent.getEditPanel().show(widgetComponent);' +
			'}' +
		'}';
		data.onComboChange = 'function(itemInfo) {' +
			'Ext.getCmp("' + this.getId() + '").changeLocale(itemInfo.value);' +
		'}';

		return data;
	}
});