/**
 * @class Sb.widget.ScriptEditPanel
 * @extends Sb.widget.WidgetEditPanel
 * Class for script-based widgets
 */
Ext.define('Sb.widget.ScriptEditPanel', {
	extend: 'Sb.widget.WidgetEditPanel',

	initComponent: function () {
		this.initEditComponents();
		Ext.applyIf(this, {
			layout		: 'form',
			labelAlign	: 'top',
			autoHeight	: true,
			itemCls		: 'sb-widget-edit-panel-script-form',
			cls			: 'sb-widget-edit-panel-script',
			items		: [this.widgetLabel]
		});
		Sb.widget.ScriptEditPanel.superclass.initComponent.apply(this, arguments);
		this.addItems();
	},

	/**
	 * Initiates edit components
	 * @private
	 */
	initEditComponents: function () {
		this.widgetLabel = new Ext.form.Label();
		this.codeTextArea = new Ext.form.TextArea({
			width		: 545,
			height		: 150,
			hideLabel	: true,
			listeners	: {
				scope	: this,
				render	: function () {
					this.el.set({
						spellcheck: false
					});
				}
			}
		});
	},

	/**
	 * Add items in edit panel
	 */
	addItems: function () {
		this.items.add(this.codeTextArea);
	},

	show: function (widget) {
		Sb.widget.ScriptEditPanel.superclass.show.call(this, widget);
		this.codeTextArea.setRawValue(this.widget.model.data.code);
		this.widgetLabel.setText(widget.getType().getTranslation('codeLabel') + ' <a href="' +
			Sb.Editor.getHelpUrl('Widgets/' + widget.xtype.toUpperCaseFirst()) +
			'" target="_blank">' + Sb.Locale.get('learnMore') + '</a>', false);
	}
});