/**
 * @class Sb.widget.Slider
 * @extends Sb.widget.Widget
 * Class for slider widget
 * @xtype slider_widget
 */
Ext.define('Sb.widget.Slider', {
	extend: 'Sb.widget.Widget',
	xtype: 'slider_widget',
	translatable: true,

	cls: 'sb-widget-slider',

	onRender: function () {
		Sb.widget.Slider.superclass.onRender.apply(this, arguments);

		var resourcesUrl = Sb.Config.siteResourcesUrl;
		Sb.ResourceManager.require([
			resourcesUrl + '/components/' + Sb.Config.jsFrameworkFile
		], function () {
			Sb.ResourceManager.require([
				resourcesUrl + '/components/jquery/nivo-slider/themes/default/default.css',
				resourcesUrl + '/components/jquery/nivo-slider/nivo-slider.css',
				resourcesUrl + '/components/jquery/nivo-slider/jquery.nivo.slider.js'
			], this.refresh, this);
		}, this);
	},

	initEvents: function () {
		Sb.widget.Slider.superclass.initEvents.call(this);

		this.mon(this.model, 'change', this.refresh, this);
	},

	onStartEdit: function () {
		Sb.widget.Slider.superclass.onStartEdit.call(this);
		if (this.nivoSlider) {
			this.nivoSlider.stop();
		}
	},

	onCompleteEdit: function () {
		Sb.widget.Slider.superclass.onCompleteEdit.apply(this, arguments);
		this.refresh();
	},

	onCancelEdit: function () {
		Sb.widget.Slider.superclass.onCancelEdit.call(this);
		Sb.SliderWidgetService.syncFiles(this.model.get('uuid'));
	},

	refresh: function () {
		if (this.nivoSlider) {
			this.nivoSlider.destroy();
		}

		this.togglePlaceholder(!this.model.isConfiguredProperties());
		if (this.model.isConfiguredProperties()) {
			this.renderSlider();
		} else {
			this.renderPlaceholder();
		}

		this.reanchorEditPanel();
	},

	renderSlider: function () {
		var imageHtml,
			imagesHtml = '',
			nivoSliderEl,
			wrapCls;

		this.model.images.each(function(modelImage) {
			imageHtml = '<img alt=""'
				+ ' src="' + this.model.getResourcesUrl()
				+ '/' + modelImage.get('image')
				+ '" title="' + Ext.util.Format.htmlEncode(modelImage.get('description')) + '">';


			imagesHtml += imageHtml;
		}, this);

		wrapCls = 'slider-wrapper theme-default';
		if (this.model.get('navigation') == 'inside') {
			wrapCls += ' widget-slider-navigation-inside';
		}

		this.update(
			'<div class="' + wrapCls + '"><div class="nivoSlider">' + imagesHtml + '</div></div>'
		);

		nivoSliderEl = jQuery('#' + this.widgetContentEl.id + ' .nivoSlider');
		nivoSliderEl.nivoSlider({
			effect: this.model.get('effect'),
			animSpeed: 400,
			pauseTime: this.model.get('pauseTime'),
			directionNav: this.model.get('arrows'),
			controlNav: this.model.get('navigation') != 'none'
		});
		this.nivoSlider = nivoSliderEl.data('nivoslider');
	},

	renderPlaceholder: function () {
		Sb.widget.Slider.superclass.renderPlaceholder.apply(this, arguments);
		var type = this.getType();
		this.update(this.placeholderTpl.apply({
			placeholderMsg: type.getTranslation('descriptionDisabled' + type.disableReason.toUpperCaseFirst())
		}));
		this.addClass(this.getStateCls('inactive'));
	},

	onDestroy: function () {
		if (this.nivoSlider) {
			this.nivoSlider.destroy();
		}
		Sb.widget.Slider.superclass.onDestroy.call(this);
	},

	onStylesChange: function (styles) {
		this.callParent(arguments);

		if (this.widgetEl && styles.boxShadow && Sb.Utils.isInnerShadow(styles.boxShadow)) {
			this.widgetEl.setStyle('boxShadow', '');
		}

		if (styles.borderRadius) {
			Sb.StyleManager.addStyleSheet(
				'#widget-' + this.model.data.uuid + ' .nivoSlider {border-radius: ' + styles.borderRadius + '}',
				'widget-slider',
				this.model.data.uuid
			);
		}
	}
});