/**
 * @class Sb.widget.VideoEditPanel
 * @extends Sb.widget.ScriptEditPanel
 */
Ext.define('Sb.widget.VideoEditPanel', {
	extend: 'Sb.widget.ScriptEditPanel',

	cls: 'sb-widget-edit-panel-script sb-widget-edit-panel-video',

	/**
	 * Initiates edit components
	 * @private
	 */
	initEditComponents: function () {
		Sb.widget.VideoEditPanel.superclass.initEditComponents.call(this);
		this.codeTextArea.setHeight(56);
		this.codeTextArea.setWidth('100%');

		this.urlText	= new Ext.form.TextField({
			width		: '100%',
			hideLabel	: true,
			cls			: 'sb-widget-video-url-provider sb-widget-video-url-provider-unknown',
			listeners	: {
				valid: function () {
					var widget	= this.widget,
						model	= widget.model;

					model.unserialize({
						defaultWidth: widget.widgetContentEl.getWidth(true),
						url			: this.urlText.getValue().trim()
					});
				},
				specialkey: function (field, e) {
					if (e.getKey() == e.ENTER) {
						var widget = this.widget;
						if (widget.model.currentProvider.isKnown) {
							widget.completeEdit();
						}
					}
				},
				scope: this
			}
		});

		this.urlRadio	= new Ext.form.Radio({
			boxLabel	: Sb.Locale.get('videoWidgetSelectOptionUrl'),
			hideLabel	: true,
			inputValue	: 'Url',
			name		: 'selectOptions',
			itemCls		: 'sb-video-insertion-url',
			listeners	: {check: this.onTypeChange, scope: this}
		});
		this.codeRadio	= new Ext.form.Radio({
			boxLabel	: Sb.Locale.get('videoWidgetSelectOptionCustom'),
			hideLabel	: true,
			inputValue	: 'Custom',
			name		: 'selectOptions',
			itemCls		: 'sb-video-insertion-custom',
			listeners	: {check: this.onTypeChange, scope: this}
		});

		this.urlText.hide();
		this.codeTextArea.hide();
	},

	/**
	 * Add items in edit panel
	 */
	addItems: function () {
		this.urlDescription = new Ext.BoxComponent({
			html: this.getUrlDescription(),
			cls	: 'sb-editor-box-description'
		});
		this.items.add(new Ext.Container({
			layout	: 'form',
			width	: 520,
			items	: [
				this.urlRadio,
				this.urlDescription,
				this.urlText,
				this.codeRadio,
				new Ext.BoxComponent({
					html: Sb.Locale.get('videoWidgetCustomDescription'),
					cls	: 'sb-editor-box-description'
				}),
				this.codeTextArea
			]
		}));
	},

	/**
	 * Returns video widget URL description with providers, that allow uploads.
	 *
	 * @returns String
	 */
	getUrlDescription: function () {
		var youtubeMatch = Sb.Config.gettingStartedDemoUrl.match(/v\/(.*)\?/),
			videoUrl = Sb.Config.gettingStartedDemoEnabled && youtubeMatch
			? 'https://youtube.com/watch?v=' + youtubeMatch[1] + ' '
			: 'https://www.youtube.com/watch?v=EFn2VYUGMN8',
			result		= Sb.Locale.get('videoWidgetUrlDescription', videoUrl),
			providers	= Sb.WidgetTypeManager.get('video').getProviders({isUpload: true}),
			lastIndex	= providers.length - 1;

		Ext.each(providers, function (provider, i) {
			if (provider.host == 'youtu.be') {
				return true;
			}
			if (i == lastIndex) {
				result += ' ' + Sb.Locale.get('or') + ' ';
			} else if (i > 0) {
				result += ', ';
			}
			result += '<a href="http://' + provider.host  + '/" target="_blank">' + provider.label + '</a>';
		});

		return result;
	},

	/**
	 * Fires on insert type change
	 *
	 * @param radio
	 * @param value
	 */
	onTypeChange: function (radio, value) {
		var isCustom		= 'Custom' == radio.inputValue,
			isCustomValue	= isCustom ? 1 : 0,
			currentControl	= this[isCustom ? 'codeTextArea' : 'urlText'],
			oppositeControl	= this[!isCustom ? 'codeTextArea' : 'urlText'],
			model			= this.widget.model,
			url				= model.data.url;

		if (value) {
			oppositeControl.hide();
			this.urlDescription.setVisible(!isCustom);
			model.set('isCustom', isCustomValue);
			if (!isCustom) {
				this.urlText.setValue(url);
				this.urlText.focus(null, true);
			} else {
				this.codeTextArea.focus(null, true);
			}
			currentControl.show();
			this.reanchor();
		}
	},

	/**
	 * Updates url text class to change icon in it
	 */
	updateUrlTextClass: function () {
		var provider	= this.widget.model.data.provider.replace(' ', '').toLowerCase(),
			dom			= this.urlText.el.dom,
			reReplace	= new RegExp('('  + RegExp.escape('sb-widget-video-url-provider-') + ')[^\\- ]+');

		dom.className	= dom.className.replace(reReplace, '$1' + provider);
	},

	/**
	 * Fires after url in model has changed
	 */
	afterUrlChanged: function () {
		this.updateUrlTextClass();
		this.codeTextArea.setRawValue(this.widget.model.data.code);
	},

	show: function (widget) {
		Sb.widget.VideoEditPanel.superclass.show.call(this, widget);

		var model	= this.widget.model,
			data	= model.data;

		if (data.isCustom == 0) {
			this.urlRadio.setValue(true);
			this.urlText.focus();
		} else {
			this.codeRadio.setValue(true);
			this.codeTextArea.focus();
		}
		this.urlText.setRawValue(data.url);
		this.afterUrlChanged();

		model.on('urlchange', this.afterUrlChanged, this);
		this.on('hide', function () {
			model.un('urlchange', this.afterUrlChanged, this);
		}, this, {single: true});
	}
});