/**
 * Base class for all widget edit panels
 *
 * @class Sb.widget.WidgetEditPanel
 * @extends Sb.editor.component.BaseEditPanel
 */
Ext.define('Sb.widget.WidgetEditPanel', {
	extend: 'Sb.editor.component.BaseEditPanel',

	show: function (widget) {
		this.widget = widget;
		Sb.widget.WidgetEditPanel.superclass.show.call(this, widget);
	},

	/**
	 * Binds fields of the panel to model, unversally integrating change events.
	 *
	 * @param {Array|Object} fieldsMapping Mapping of fields to model. Could be just array or object modelKey => fieldName.
	 * @param {Sb.model.Node} [model] Model for binding. Defaults to the current Widget.
	 */
	bindModel: function (fieldsMapping, model) {
		model = model || this.widget.model;
		var fields = {};

		if (Ext.isString(fieldsMapping)) {
			fieldsMapping = [fieldsMapping];
		}
		if (Ext.isArray(fieldsMapping)) {
			Ext.each(fieldsMapping, function (item) {
				fields[item] = item;
			});
		} else {
			fields	= fieldsMapping;
		}

		Ext.iterate(fields, function (modelKey, field) {
			var	fieldObj	= Ext.isObject(field) ? field : this[field],
				fieldXType	= fieldObj.getXType(),
				eventName	= 'valid',
				getValueFn	= 'getValue',
				setValueFn	= 'setValue',
				changeFn	= function () {
					model.set(modelKey, this[getValueFn]());
				};

			if ('checkbox' == fieldXType) {
				eventName	= 'check';
			} else if ('button' == fieldXType) {
				if (!fieldObj.enableToggle) {
					return;
				}
				eventName	= 'toggle';
				getValueFn	= 'getPressed';
				setValueFn	= 'toggle';
			} else if (
				['colordropdown', 'expander', 'spinnerfield'].indexOf(fieldXType) > -1
			) {
				eventName = 'change';
			} else if ('pixelfield' === fieldXType) {
				eventName = 'change';
				changeFn = function () {
					if (this.isValid(true)) {
						model.set(modelKey, this[getValueFn]());
					}
				};
			} else if ('radiogroup' == fieldXType) {
				changeFn	= function () {
					var value = this.getValue();
					if (null != value) {
						value = value.getGroupValue();
					}
					model.set(modelKey, value);
				};
			} else if ('combo' == fieldXType) {
				if (fieldObj.editable) {
					eventName	= 'valid';
					getValueFn	= 'getRawValue';
				} else {
					eventName	= 'select';
				}
			} else if ('sbfontfamilycombo' == fieldXType || 'sbselectbox' == fieldXType ||
				fieldXType == 'sbshadowselectbox') {
				eventName = 'select';
			}

			fieldObj[setValueFn](model.get(modelKey));
			fieldObj.on(eventName, changeFn);
			this.on('hide', function () {
				fieldObj.un(eventName, changeFn);
			}, this, {single: true});
		}, this);
	}
});