/**
 * @class Ext.ux.jsonp.Manager
 * <p>Allows us to do jsonp requests with callbacks.</p>
 * @singleton
 */
Ext.define('Ext.ux.jsonp.Manager', function () {
	var queue		= [],
		timer		= new Ext.util.DelayedTask(this.callback, this),
		isActive	= false;

	function processQueue() {
		if (isActive || queue.length === 0) {
			return;
		}

		var request = queue[0];
		request.start('Ext.ux.jsonp.Manager.callback');

		timer.delay(request.timeout);
		isActive = true;
	}

	return {
		singleton: true,

		/**
		 * Creates new request by passed options
		 *
		 * @param {Object} options
		 */
		request: function (options) {
			queue.push(
				new Ext.ux.jsonp.Request(options)
			);
			processQueue();
		},

		/**
		 * Finishes previously created request
		 *
		 * @param {Object} json
		 */
		callback: function (json) {
			timer.cancel();
			isActive = false;

			var request = queue.shift();
			request.finish(json);
			Ext.destroy(request);

			processQueue();
		}
	};
});