/**
 * @class Ext.ux.uploader.Manager
 * @extends Ext.util.Observable
 * @constructor
 */
Ext.define('Ext.ux.uploader.Manager', {
	extend: 'Ext.util.Observable',

	constructor: function (config) {
		this.addEvents(
			'startupload',
			'completeupload',
			'itemuploadsuccess',
			'itemuploadfailure'
		);

		Ext.apply(this, {
			url: config.url,
			listeners: config.listeners,
			waitMsg: config.waitMsg,
			items: new Ext.util.MixedCollection(),
			uploadActive: false,
			errorCount: 0
		});

		if (config.items) {
			this.addItems(config.items);
		}

		this.uploader = new Ext.ux.uploader.Uploader({
			url: this.url,
			params: config.params || {}
		});

		this.uploader.on('uploadsuccess', this.onUploadSuccess, this);
		this.uploader.on('uploadfailure', this.onUploadFailure, this);
		Ext.ux.uploader.Manager.superclass.constructor.call(this, config);
	},

	startUpload: function () {
		if (this.uploadActive || this.items.length === 0) {
			return;
		}

		this.uploadActive = true;
		this.fireEvent('startupload', this);

		if (this.waitMsg) {
			Ext.MessageBox.wait(this.waitMsg, this.waitTitle);
		}

		this.uploadNextItem();
	},

	uploadNextItem: function () {
		if (!this.uploadActive) {
			return;
		}

		var item;
		this.items.each(function (i) {
			if (i.isReady()) {
				item = i;
				return false;
			}
		});

		if (!item) {
			return;
		}

		item.setUploading();
		this.uploader.uploadItem(item);
	},

	finishUpload: function () {
		this.fireEvent('completeupload', this, this.errorCount);
		if (this.waitMsg) {
			Ext.MessageBox.updateProgress(1);
			Ext.MessageBox.hide();
		}
		this.uploadActive = false;
		this.errorCount = 0;
	},

	/**
	 * @param {FileList} fileList
	 */
	addItems: function (fileList) {
		if (!fileList.length) {
			return;
		}

		var items = [];
		Ext.each(fileList, function (file) {
			var item = new Ext.ux.uploader.Item({
				file: file
			});
			items.push(item);
		}, this);

		this.items.addAll(items);
	},

	clearItems: function () {
		this.items.clear();
	},

	/**
	 * @return {Ext.ux.uploader.Item[]}
	 */
	getItems: function () {
		return this.items.getRange();
	},

	afterItemUpload: function (item) {
		this.uploadNextItem();

		if (this.items.last().getFilename() === item.getFilename()) {
			this.finishUpload();
		}
	},

	onUploadSuccess: function (item, info) {
		item.setUploaded();
		this.fireEvent('itemuploadsuccess', item, info);
		this.afterItemUpload(item);
	},

	onUploadFailure: function (item, info) {
		item.setUploadError(info.message);
		this.errorCount++;
		this.fireEvent('itemuploadfailure', item, info);
		this.afterItemUpload(item);
	}

});